#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "vpu2image4xi.py">
#<abstract4xi abstract = "VPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "vpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'VPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4vpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4vpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4vpu',              # ACK 5.3.4 p92
            'create2image4vpu',                    # ACK 5.31  p76            

            'get2imageinfo4vpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4vpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4vpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4vpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4vpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4vpu',     # ACK modernized
            'manoeuvre2mapimage4vpu',              # ACK modernized                                           
            'manoeuvre2readimage4vpu',             # ACK modernized
            'manoeuvre2writeimage4vpu',            # ACK modernized            
            'map2image4vpu',                       # ACK 5.3.5 p94 
            
            'read2image4vpu',                      # ACK 5.3.3 p84  
            'retain2image4vpu',                    # ACK 5.4.1 p99
            'release2image4vpu',                   # ACK 5.4.1 p99

            'set2imagecallback4vpu',               # ACK modernized
            'set2image4vpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4vpu',                     # ACK modernized 
                                    
            'wait2image4vpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4vpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from vpu2buffer4xi import VPU2Buffer4Xi
from vpu2buffer4xi import create2buffer4vpu
from vpu2buffer4xi import create2subbuffer4vpu
from vpu2buffer4xi import get2bufferinfo4vpu
from vpu2buffer4xi import release2buffer4vpu
from vpu2buffer4xi import retain2buffer4vpu
from vpu2buffer4xi import set2buffer4vpu
from vpu2buffer4xi import trace2buffer4vpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from vpu2commandqueue4xi import VPU2CommandQueue4Xi
from vpu2commandqueue4xi import create2commandqueue4vpu 
from vpu2commandqueue4xi import flush2commandqueue4vpu
from vpu2commandqueue4xi import finish2commandqueue4vpu
from vpu2commandqueue4xi import get2commandqueueinfo4vpu
from vpu2commandqueue4xi import release2commandqueue4vpu
from vpu2commandqueue4xi import retain2commandqueue4vpu
from vpu2commandqueue4xi import set2commandqueue4vpu
from vpu2commandqueue4xi import trace2commandqueue4vpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from vpu2context4xi import VPU2Context4Xi 
from vpu2context4xi import create2context4vpu
from vpu2context4xi import get2context4vpu
from vpu2context4xi import release2context4vpu
from vpu2context4xi import retain2context4vpu
from vpu2context4xi import set2context4vpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from vpu2device4xi import VPU4Xi
from vpu2device4xi import init2vpu4xi 
from vpu2device4xi import set2vpu4xi
from vpu2device4xi import get2vpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from vpu2event4xi import VPU2Event4Xi
from vpu2event4xi import create2event4vpu
from vpu2event4xi import get2eventinfo4vpu
from vpu2event4xi import release2event4vpu
from vpu2event4xi import retain2event4vpu
from vpu2event4xi import set2eventcallback4vpu 
from vpu2event4xi import set2event4vpu
from vpu2event4xi import trace2event4vpu
from vpu2event4xi import wait2event4vpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global vpu2image4xi                                                     # 2D-IMAGES FOR VPU

global vpu2trace4xi                                                     # TRACE

global vpu2buffer4xi                                                    # BUFFER FOR VPU

global control2readimage4vpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH VPU  
global state2read4vpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4vpu                                                # MAX FOR STATE
global ack2readinfo4vpu                                                 # ACK FOR IMAGE INFO
global message2readget4vpu                                              # QUEUE GET
global message2readput4vpu                                              # QUEUE PUT
global message2readreceive4vpu                                          # PIPE RECEIVE
global message2readsend4vpu                                             # PIPE SEND

global control2writeimage4vpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH VPU 
global state2write4vpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4vpu                                               # MAX FOR STATE
global ack2writeinfo4vpu                                                # ACK FOR IMAGE INFO
global message2writeget4vpu                                             # QUEUE GET
global message2writeput4vpu                                             # QUEUE PUT
global message2writereceive4vpu                                         # PIPE RECEIVE
global message2writesend4vpu                                            # PIPE SEND

global control2copyimage4vpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH VPU 
global state2copyimage4vpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4vpu                                           # MAX FOR STATE
global ack2copyimageinfo4vpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4vpu                                         # QUEUE GET
global message2copyimageput4vpu                                         # QUEUE PUT
global message2copyimagereceive4vpu                                     # PIPE RECEIVE
global message2copyimagesend4vpu                                        # PIPE SEND

global control2copyimagetobuffer4vpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH VPU 
global state2copyimagetobuffer4vpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4vpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4vpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4vpu                                 # QUEUE GET
global message2copyimagetobufferput4vpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4vpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4vpu                                # PIPE SEND

global control2copybuffertoimage4vpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH VPU 
global state2copybuffertoimage4vpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4vpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4vpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4vpu                                 # QUEUE GET
global message2copybuffertoimageput4vpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4vpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4vpu                                # PIPE SEND                      

global control2mapimage4vpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH VPU  
global state2map4vpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4vpu                                                # MAX FOR STATE
global ack2mapinfo4vpu                                                 # ACK FOR IMAGE INFO
global message2mapget4vpu                                              # QUEUE GET
global message2mapput4vpu                                              # QUEUE PUT
global message2mapreceive4vpu                                          # PIPE RECEIVE
global message2mapsend4vpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4vpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF VPU2IMAGE4XI5TEST - TRYING TO TRACE VPU: ', time4xi())      
#   print('=' *33)      
#   trace2vpu = None
#   try:
#       trace2vpu = trace2image4vpu() 
#       print('(Step 15B/25) OF VPU2IMAGE4XI5TEST - TRACE2VPU: ', trace2vpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4vpu', 'METHOD': 'trace2image4vpu'}
#   except:
#       print('(Step 15Z1/25) OF VPU2IMAGE4XI5TEST *** FAILED TO TRACE VPU: ', trace2vpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4vpu(task4trace2vpu=None):  

    '''
    creates callback to vpu2image4xi
    '''
    
    global vpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4vpu" 
    method2print = "trace2image4vpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4vpu-started: " + str(trace2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4vpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2vpu-to-trace2image4vpu: " + str(task4trace2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    vpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2vpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY VPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2vpu = {}                                                                                            
            
        task4trace2vpu['CLASS']   = "trace2image4vpu"                                                               # VPUIZE ZPU TRACE!
        task4trace2vpu['METHOD']  = "trace2image4vpu" 
        task4trace2vpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4vpu-failed " + str(vpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2vpu-set-by-trace2image4vpu to: " + str(task4trace2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4vpu',
        #  'METHOD': 'trace2image4vpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY VPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2vpu-try-by-trace2image4vpu: " + str(task4trace2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4vpu : trace2image4vpu ; (Step 3A/33) try; task4trace2vpu-try-by-trace2image4vpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4vpu',
    #  'METHOD': 'trace2image4vpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        vpu2trace4xi ['CLASS']  = task4trace2vpu['CLASS']                                                      # VPUIZE VPU TRACE!
        vpu2trace4xi ['METHOD']  = task4trace2vpu['METHOD']
        vpu2trace4xi ['MESSAGE'] = task4trace2vpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; vpu2trace4xi-set-by-trace2image4vpu to: " + str(vpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4vpu : trace2image4vpu ; (Step 3B/33) set; vpu2trace4xi-set-by-trace2image4vpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4vpu',
        #                      'METHOD': 'trace2image4vpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; vpu-trace-failed-by-trace2image4vpu: " + str(task4trace2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE VPU TRACE
        # =================================
        class2print   = task4trace2vpu['CLASS']                                                                # VPUIZE VPU TRACE!
        method2print  = task4trace2vpu['METHOD']
        
        message2print = task4trace2vpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # VPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4vpu : trace2image4vpu ; (Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4vpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4vpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-vpu-trace-failed-by-trace2image4vpu: " + str(task4trace2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4vpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4vpu"                
        message2print = "{(Step 3D/33) performed" + "; vpu-trace-performed-by-trace2image4vpu: " + str(task4trace2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4vpu : trace2image4vpu ; (Step 3D/33) performed; vpu-trace-performed-by-trace2image4vpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4vpu', 'METHOD': 'trace2image4vpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; vpu-trace-failed-by-trace2image4vpu: " + str(task4trace2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; vpu2trace4xi-returning-by-trace2image4vpu: " + str(vpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4vpu', 
    #  'METHOD': 'trace2image4vpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return vpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4vpu" usage = "
#
#   print('(Step 14A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF VPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4vpu()                                                                                                 
#       print('(Step 14B/700) OF VPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF VPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4vpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF VPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4vpu(task4set2imagecallback=None):

    '''
    registers callback function to vpu2image4xi
    '''
    global vpu2image4xi
    
    class2print = "set2imagecallback4vpu" 
    method2print = "set2imagecallback4vpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4vpu-started: " + str(set2imagecallback4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4vpu at 0x000000000390B748>
    
    try:
    
        vpu2image4xi ['CALLBACK'] = trace2image4vpu
        
        message2print = "{(Step 1B/70) set callback2vpu" + "; imagecallback4vpu-set-by-set2imagecallback4vpu: " + str(vpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4vpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-vpu2image4xi-returning-by-set2imagecallback4vpu: " + str(vpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4vpu at 0x0000000003864A48> 
        
        return vpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4vpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4vpu-returning-by-set2imagecallback4vpu: " + str(trace2image4vpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4vpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4vpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF VPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4VPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4vpu()                                                                 
#       print('(Step 2B/700) OF VPU2IMAGE4XI5TEST - CREATED 2D IMAGE4VPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4VPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF VPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for VPU ', time4xi()) 
#   # 
#   try:
#       xi2vpu = VPU2Image4Xi()                            
#       response4create2Dimage = xi2vpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF VPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for VPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4vpu(task4imagecreate2vpu=None):

    '''
    creates 2D image for VPU
    '''
    
    global vpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4vpu" 
    method2print = "create2image4vpu"

    message2print = "{(Step 1A/70) started" + "; create2image4vpu-started: " + str(create2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4vpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2vpu-to-create2image4vpu: " + str(task4imagecreate2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    vpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2vpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO VPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2vpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2vpu-created-by-create2image4vpu to: " + str(task4imagecreate2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4vpu = trace2image4vpu                                                                 # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4vpu-created-by-create2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4vpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4vpu"                                                          # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4vpu"                                                          # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4vpu',
            #  'METHOD': 'create2image4vpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2vpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2vpu-created-by-create2image4vpu to: " + str(task4imagecreate2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4vpu',
            #  'METHOD': 'create2image4vpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2vpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-create2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-create2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4vpu',
                #  'METHOD': 'create2context4vpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-create2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2vpu ['CONTEXT'] = context2vpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-create2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-create2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_VPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-create2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2vpu ['DEVICE'] = device2vpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2vpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2vpu-trying-by-create2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2vpu-got-by-create2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2vpu-failed-by-create2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2vpu ['QUEUE'] = queue2vpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2vpu-trying-by-create2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2vpu-got-by-create2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2vpu-failed-by-create2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2vpu ['PIPE'] = pipe2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2vpu-trying-by-create2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2vpu-got-by-create2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4vpu',
                #  'METHOD': 'create2commandqueue4vpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4vpu', 
                #  'METHOD': 'create2context4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2vpu-failed-by-create2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2vpu-trying-by-create2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                                   # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2vpu-got-by-create2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4vpu',
                #                    'METHOD': 'create2buffer4vpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4vpu', 
                #  'METHOD': 'create2context4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2vpu-failed-by-create2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2vpu ['BUFFER'] = bufferinfo2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2vpu-trying-by-create2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2vpu-got-by-create2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4vpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4vpu',
                #                    'METHOD': 'create2event4vpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4vpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4vpu', 'METHOD': 'create2context4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2vpu-failed-by-create2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2vpu ['EVENT'] = eventinfo2vpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2vpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2vpu-trying-by-create2image4vpu: " + str(flags2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2vpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2vpu-set-by-create2image4vpu to: " + str(flags2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2vpu-failed-by-create2image4vpu: " + str(flags2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2vpu ['FLAGS'] = flags2vpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2vpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2vpu-trying-by-create2image4vpu: " + str(width2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2vpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2vpu = 2 * width2vpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2vpu-set-by-create2image4vpu to: " + str(width2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2vpu-failed-by-create2image4vpu: " + str(width2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2vpu ['WIDTH'] = width2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2vpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2vpu-trying-by-create2image4vpu: " + str(height2vpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2vpu = height2layer * quantity2total
                
                height2vpu = 2 * height2vpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2vpu-set-by-create2image4vpu to: " + str(height2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2vpu-failed-by-create2image4vpu: " + str(height2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2vpu ['HEIGHT'] = height2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2vpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2vpu-trying-by-create2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2vpu-set-by-create2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2vpu-failed-by-create2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2vpu ['ROWPITCH'] = rowpitch2vpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2vpu = None
            size2vpu = int(width2vpu * height2vpu)                                                                  # CALCULATED MEMORY
            if size2vpu == 0:
                size2vpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2vpu-trying-by-create2image4vpu: " + str(pointer2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2vpu = Arena4Xi(size2vpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2vpu-got-by-create2image4vpu: " + str(arena2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2vpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4vpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2vpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4vpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2vpu:
                    size2vpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2vpu-set-by-create2buffer4vpu to: " + str(size2vpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2vpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2vpu-set-by-create2buffer4vpu to: " + str(pointer2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2vpu-failed-by-create2image4vpu: " + str(pointer2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2vpu ['POINTER'] = pointer2vpu                            
            task4imagecreate2vpu ['SIZE']    = size2vpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2vpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2vpu-trying-by-create2image4vpu: " + str(format2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2vpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2vpu-set-by-create2image4vpu to: " + str(format2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2vpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2vpu-set-by-create2image4vpu to: " + str(format2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2vpu-failed-by-create2image4vpu: " + str(format2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2vpu ['FORMAT'] = format2vpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2vpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2vpu-set-by-create2image4vpu to: " + str(depth2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2vpu ['DEPTH'] = depth2vpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2vpu-yielded-by-create2image4vpu: " + str(task4imagecreate2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2vpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR VPU! 
    # ---------------------------------------------------------------------------------------------    
    vpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; vpu2image4xi-initialized-by-create2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['DATA2CALLBACK'] = task4imagecreate2vpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4vpu to: " + str(vpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4vpu',
        #  'METHOD': 'create2image4vpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2vpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4vpu to: " + str(vpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4vpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['PLATFORM'] = task4imagecreate2vpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4vpu to: " + str(vpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2vpu-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['CONTEXT'] = task4imagecreate2vpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4vpu to: " + str(vpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4vpu',
        #  'METHOD': 'create2context4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['DEVICE'] = task4imagecreate2vpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4vpu to: " + str(vpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['CODE2ERROR'] = task4imagecreate2vpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4vpu to: " + str(vpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['QUEUE'] = task4imagecreate2vpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4vpu to: " + str(vpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['PIPE'] = task4imagecreate2vpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4vpu to: " + str(vpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-vpu2image4xi-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['COMMANDQUEUE']  = task4imagecreate2vpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4vpu to: " + str(vpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4vpu',
        #  'METHOD': 'create2commandqueue4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['BUFFER']  = task4imagecreate2vpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4vpu to: " + str(vpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4vpu',
        #  'METHOD': 'create2buffer4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['EVENT']  = task4imagecreate2vpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4vpu to: " + str(vpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4vpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4vpu',
        #                     'METHOD': 'create2event4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4vpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['FLAGS']  = task4imagecreate2vpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4vpu to: " + str(vpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        vpu2image4xi['WIDTH']  = task4imagecreate2vpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4vpu to: " + str(vpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        vpu2image4xi['HEIGHT']  = task4imagecreate2vpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4vpu to: " + str(vpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['ROWPITCH']  = task4imagecreate2vpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4vpu to: " + str(vpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4vpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['POINTER']  = task4imagecreate2vpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4vpu to: " + str(vpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['FORMAT']  = task4imagecreate2vpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4vpu to: " + str(vpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['DEPTH']  = task4imagecreate2vpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4vpu to: " + str(vpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    vpu2image4xi ['ACK2RETURN']  = code2ack
    vpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; vpu2image4xi-returning-by-create2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return vpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4vpu" usage = "
#    
#   print('(Step 4B/33) OF VPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4VPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4VPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4VPU5TEST - TRYING TO INITIALIZE DEVICE4VPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2vpu = init2vpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4VPU5TEST - INITIALIZED DEVICE4VPU TO: ', device2vpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4VPU5TEST *** FAILED TO INITIALIZE DEVICE4VPU', time4xi()) 
#   print('(Step 3A/33) OF VPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4VPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2vpu = create2context4vpu()                                                                               
#       print('(Step 3B/33) OF VPU2IMAGE4XI5TEST - CREATED CONTEXT4VPU: ', context2vpu)                                  
#   except:
#       print('(Step 3Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4VPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2vpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4VPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4vpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF VPU2IMAGE4XI5TEST - SET 2D IMAGE4VPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4VPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4vpu(task4imageset2vpu=None):

    '''
    sets 2D images for VPU
    '''
    global vpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4vpu"
    method2print = "set2image4vpu"

    message2print = "{(Step 1A/33) started" + "; set2image4vpu-started: " + str(set2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4vpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2vpu-to-set2image4vpu: " + str(task4imageset2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4vpu',
    #              'METHOD': 'create2context4vpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4vpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4vpu',
    #              'METHOD': 'create2commandqueue4vpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4vpu',
    #              'METHOD': 'create2context4vpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4vpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) set" + "; vpu2image4xi-set-by-set2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-set2image4vpu: " + str(create2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4vpu at 0x0000000003A627C8>;                                  
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-set2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['DATA2CALLBACK'] = task4imageset2vpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4vpu to: " + str(vpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2vpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4vpu to: " + str(vpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['PLATFORM'] = task4imageset2vpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4vpu to: " + str(vpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2vpu-try-by-set2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['CONTEXT'] = task4imageset2vpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4vpu to: " + str(vpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4vpu',
        #                    'METHOD': 'create2context4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['DEVICE'] = task4imageset2vpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4vpu to: " + str(vpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_VPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['CODE2ERROR'] = task4imageset2vpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4vpu to: " + str(vpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['QUEUE'] = task4imageset2vpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4vpu to: " + str(vpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['PIPE'] = task4imageset2vpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4vpu to: " + str(vpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-vpu2image4xi-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['COMMANDQUEUE']  = task4imageset2vpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4vpu to: " + str(vpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['BUFFER']  = task4imageset2vpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4vpu to: " + str(vpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4vpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['EVENT']  = task4imageset2vpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4vpu to: " + str(vpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['FLAGS']  = task4imageset2vpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4vpu to: " + str(vpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        vpu2image4xi['WIDTH']  = task4imageset2vpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4vpu to: " + str(vpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4vpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        vpu2image4xi['HEIGHT']  = task4imageset2vpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4vpu to: " + str(vpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['ROWPITCH']  = task4imageset2vpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4vpu to: " + str(vpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['POINTER']  = task4imageset2vpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4vpu to: " + str(vpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4vpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        vpu2image4xi['FORMAT']  = task4imageset2vpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4vpu to: " + str(vpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    vpu2image4xi ['ACK2RETURN']  = code2ack
    vpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; vpu2image4xi-returning-by-set2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return vpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4vpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4VPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4VPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4vpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4VPU : ', response4copy2image)
#       queue2copy4vpu      = response4copy2image['QUEUE']
#       channel2copypublish4vpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4vpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4VPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2copyimagesend4vpu = "<copy4xi>copy2copy4vpu</copy4xi>"
#       channel2copypublish4vpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4vpu.send(message2copyimagesend4vpu)
#       print("(Step 17B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copypublish4vpu, " MESSAGE2SEND4VPU: " , message2copyimagesend4vpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4VPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2copyimage4vpu(queue2copy4vpu, channel2copyconsume4vpu, channel2copypublish4vpu)
#       print("(Step 18D/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2copyimage4vpu)
#   except:
#       print("(Step 18Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())     
#   try:
#       channel2copyconsume4vpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copyconsume4vpu.recv()
#       print("(Step 19B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # <roger4xi>roger2copy4vpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4VPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2copyimagesend4vpu = "<copy4xi>copy2copy4vpu</copy4xi>"
#       channel2copypublish4vpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4vpu.send(message2copyimagesend4vpu)
#       print("(Step 20B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copypublish4vpu, " MESSAGE2SEND4VPU: " , message2copyimagesend4vpu)                                                                                                
#       # <copy4xi>copy2copy4vpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4VPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2copyimage4vpu(queue2copy4vpu, channel2copyconsume4vpu, channel2copypublish4vpu)
#       print("(Step 21B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2copyimage4vpu)
#   except:
#       print("(Step 21Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4VPU ', time4xi())     
#   try:
#       channel2copyconsume4vpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copyconsume4vpu.recv()
#       print("(Step 22B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # <copy4xi>copy2copy4vpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4VPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4VPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2copyimage4vpu(queue2copy4vpu, channel2copyconsume4vpu, channel2copypublish4vpu)
#       print("(Step 25B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2copyimage4vpu)
#   except:
#       print("(Step 25Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4vpu = channel2copyconsume4vpu.recv()
#       print("(Step 26D/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4vpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4VPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2copyimage4vpu(queue2copy4vpu, channel2copyconsume4vpu, channel2copypublish4vpu)
#       print("(Step 27B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvre4vpu)
#   except:
#       print("(Step 27Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4vpu(task4imagecopy2vpu=None):

    '''
    copies 2D images for VPU
    '''
    global vpu2image4xi                                                                                             # 2D-IMAGES FOR VPU  
    global control2copyimage4vpu                                                                                    # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4vpu"
    method2print = "copy2image4vpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4vpu-started: " + str(copy2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4vpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2vpu-to-copy2image4vpu: " + str(task4imagecopy2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                       # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 0C/70) created" + "; vpu2image4xi-created-by-copy2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4vpu-skipped-by-copy2image4vpu: " + str(copy2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4vpu at 0x00000000038538C8>                                 
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 0E/70) set" + "; vpu2image4xi-set-by-copy2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; vpu2image4xi-yielded-by-copy2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2vpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2vpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2vpu-created-by-copy2image4vpu to: " + str(task4imagecopy2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                                 # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-copy2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4vpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4vpu"                                                            # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4vpu"                                                            # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4vpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4vpu',
            #  'METHOD': 'copy2image4vpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2vpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2vpu-created-by-copy2image4vpu to: " + str(task4imagecopy2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4vpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4vpu',
            #  'METHOD': 'copy2image4vpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2vpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-copy2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-copy2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-copy2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2vpu ['CONTEXT'] = context2vpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-copy2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-copy2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-copy2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2vpu ['DEVICE'] = device2vpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2vpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2vpu-trying-by-copy2image4vpu: " + str(imagesourceinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2vpu-got-by-copy2image4vpu: " + str(imagesourceinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2vpu-failed-by-copy2image4vpu: " + str(imagesourceinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2vpu ['IMAGESOURCE'] = imagesourceinfo2vpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2vpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2vpu-trying-by-copy2image4vpu: " + str(imagedestinationinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2vpu-got-by-copy2image4vpu: " + str(imagedestinationinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2vpu-failed-by-copy2image4vpu: " + str(imagedestinationinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2vpu ['IMAGEDESTINATION'] = imagedestinationinfo2vpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2vpu-trying-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2vpu-got-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2vpu-failed-by-copy2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2vpu ['QUEUE'] = queue2vpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2vpu-trying-by-copy2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2vpu-got-by-copy2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2vpu-failed-by-copy2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2vpu ['PIPE'] = pipe2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2vpu-got-by-copy2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4vpu', 
                #  'METHOD': 'create2commandqueue4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2vpu-trying-by-copy2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                               # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2vpu-got-by-copy2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2vpu-failed-by-copy2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2vpu ['BUFFER'] = bufferinfo2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2vpu-trying-by-copy2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2vpu-got-by-copy2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4vpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4vpu',
                #  'METHOD': 'create2event4vpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4vpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2vpu-failed-by-copy2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2vpu ['EVENT'] = eventinfo2vpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2vpu-set-by-copy2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2vpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2vpu-trying-by-copy2image4vpu: " + str(origin4source2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2vpu['X'] = 0 
                origin4source2vpu['Y'] = 0  
                origin4source2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2vpu-set-by-copy2image4vpu to: " + str(origin4source2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2vpu-failed-by-copy2image4vpu: " + str(origin4source2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2vpu ['SOURCEORIGIN'] = origin4source2vpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2vpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2vpu-trying-by-copy2image4vpu: " + str(origin4destination2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2vpu['X'] = 100 
                origin4destination2vpu['Y'] = 100  
                origin4destination2vpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2vpu-set-by-copy2image4vpu to: " + str(origin4destination2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2vpu-failed-by-copy2image4vpu: " + str(origin4destination2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2vpu ['DESTINATIONORIGIN'] = origin4destination2vpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2vpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2vpu-trying-by-copy2image4vpu: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2vpu['X'] = 10 
                region4copy2vpu['Y'] = 10  
                region4copy2vpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2vpu-set-by-copy2image4vpu to: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2vpu-failed-by-copy2image4vpu: " + str(region4copy2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2vpu ['REGION'] = region4copy2vpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2vpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                            # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-copy2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4vpu"                                                        # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4vpu"                                                        # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2vpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2vpu-created-by-copy2image4vpu to: " + str(task4imagecopy2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2vpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-copy2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-copy2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-copy2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2vpu ['CONTEXT'] = context2vpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-copy2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-copy2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-copy2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2vpu ['DEVICE'] = device2vpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2vpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2vpu-trying-by-copy2image4vpu: " + str(sourceimageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2vpu-got-by-copy2image4vpu: " + str(sourceimageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2vpu-failed-by-copy2image4vpu: " + str(sourceimageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2vpu ['IMAGESOURCE'] = sourceimageinfo2vpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2vpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2vpu-trying-by-copy2image4vpu: " + str(destinationimageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2vpu-got-by-copy2image4vpu: " + str(destinationimageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2vpu-failed-by-copy2image4vpu: " + str(destinationimageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2vpu ['IMAGEDESTINATION'] = destinationimageinfo2vpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2vpu-trying-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2vpu-got-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2vpu-failed-by-copy2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2vpu ['QUEUE'] = queue2vpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2vpu-trying-by-copy2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2vpu-got-by-copy2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2vpu-failed-by-copy2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2vpu ['PIPE'] = pipe2vpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2vpu-got-by-copy2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2vpu-trying-by-copy2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                           # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2vpu-got-by-copy2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2vpu-failed-by-copy2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2vpu ['BUFFER'] = bufferinfo2vpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2vpu-trying-by-copy2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2vpu-got-by-copy2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2vpu-failed-by-copy2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2vpu ['EVENT'] = eventinfo2vpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2vpu-set-by-copy2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2vpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2vpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2vpu-trying-by-copy2image4vpu: " + str(source4origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2vpu['X'] = 0 
                source4origin2vpu['Y'] = 0  
                source4origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2vpu-set-by-copy2image4vpu to: " + str(source4origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2vpu-failed-by-copy2image4vpu: " + str(source4origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2vpu ['SOURCEORIGIN'] = source4origin2vpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2vpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2vpu-trying-by-copy2image4vpu: " + str(destination4origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2vpu['X'] = 100 
                destination4origin2vpu['Y'] = 100  
                destination4origin2vpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2vpu-set-by-copy2image4vpu to: " + str(destination4origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2vpu-failed-by-copy2image4vpu: " + str(destination4origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2vpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2vpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2vpu-trying-by-copy2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2vpu-set-by-copy2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2vpu-failed-by-copy2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2vpu ['REGION'] = region2vpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2vpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2vpu-yielded-by-copy2image4vpu: " + str(task4imagecopy2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-copy2image4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2vpu, publisherchannel2vpu = task4imagecopy2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2vpu-got-by-copy2image4vpu: " + str(consumerchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2vpu-got-by-copy2image4vpu: " + str(publisherchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2vpu = True
        
        control2copyimage4vpu['CONSUMERCHANNEL']  = consumerchannel2vpu
        control2copyimage4vpu['PUBLISHERCHANNEL'] = publisherchannel2vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2vpu = False
    queue2vpu = None
    message2print = "{(Step 41A/70) trying" + "queue2vpu-trying-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2vpu = task4imagecopy2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2vpu-got-by-copy2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2vpu = True
        control2copyimage4vpu['QUEUE']  = queue2vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-copy2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2copyimage4vpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2vpu, publisherchannel2vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-copy2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4vpu-returning-by-copy2image4vpu: " + str(control2copyimage4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4vpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4vpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4VPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4VPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4vpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4VPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4vpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4vpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4vpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4VPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4vpu = "<copy4xi>copy2copyimagetobuffer4vpu</copy4xi>"
#       channel2copyimagetobufferpublish4vpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4vpu.send(message2copyimagetobuffersend4vpu)
#       print("(Step 61B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copyimagetobufferpublish4vpu, " MESSAGE2SEND4VPU: " , message2copyimagetobuffersend4vpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copyimagetobuffer4vpu(queue2copyimagetobuffer4vpu, channel2copyimagetobufferconsume4vpu, channel2copyimagetobufferpublish4vpu)
#       print("(Step 62B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copyimagetobuffer4vpu)
#   except:
#       print("(Step 62Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4vpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copyimagetobufferconsume4vpu.recv()
#       print("(Step 63B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyimagetobufferconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4VPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4vpu = "<copy4xi>copy2copyimagetobuffer4vpu</copy4xi>"
#       channel2copyimagetobufferpublish4vpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4vpu.send(message2copyimagetobuffersend4vpu)
#       print("(Step 64B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copyimagetobufferpublish4vpu, " MESSAGE2SEND4VPU: " , message2copyimagetobuffersend4vpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copyimagetobuffer4vpu(queue2copyimagetobuffer4vpu, channel2copyimagetobufferconsume4vpu, channel2copyimagetobufferpublish4vpu)
#       print("(Step 65B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copyimagetobuffer4vpu)
#   except:
#       print("(Step 65Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4VPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4vpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copyimagetobufferconsume4vpu.recv()
#       print("(Step 66B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyimagetobufferconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4VPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copyimagetobuffer4vpu(queue2copyimagetobuffer4vpu, channel2copyimagetobufferconsume4vpu, channel2copyimagetobufferpublish4vpu)
#       print("(Step 68B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copyimagetobuffer4vpu)
#   except:
#       print("(Step 68Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4vpu = channel2copyimagetobufferconsume4vpu.recv()
#       print("(Step 69D/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copyimagetobufferconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copyimagetobuffer4vpu(queue2copyimagetobuffer4vpu, channel2copyimagetobufferconsume4vpu, channel2copyimagetobufferpublish4vpu)
#       print("(Step 70B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvreimagecopytobuffer4vpu)
#   except:
#       print("(Step 70Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4vpu(task4copyimagetobuffer2vpu=None):

    '''
    copies 2D images to buffer for VPU
    '''
    global vpu2image4xi                                                                                             # 2D-IMAGES FOR VPU  
    global vpu2buffer4xi                                                                                            # BUFFER FOR VPU
    
    global control2copyimagetobuffer4vpu                                                                            # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4vpu"
    method2print = "copy2imagetobuffer4vpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4vpu-started: " + str(copy2imagetobuffer4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4vpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2vpu-to-copy2imagetobuffer4vpu: " + str(task4copyimagetobuffer2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) created" + "; vpu2image4xi-created-by-copy2imagetobuffer4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-copy2imagetobuffer4vpu: " + str(copy2imagetobuffer4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4vpu-skipped-by-copy2imagetobuffer4vpu                                 
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-copy2imagetobuffer4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; vpu2image4xi-yielded-by-copy2imagetobuffer4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2buffer4xi is None:
            vpu2buffer4xi = create2buffer4vpu()                                                                     # INITIALIZE BUFFER FOR VPU 
            message2print = "{(Step 1F/70) created" + "; vpu2buffer4xi-created-by-copy2imagetobuffer4vpu to: " + str(vpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4vpu-skipped-by-copy2imagetobuffer4vpu: " + str(copy2imagetobuffer4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            vpu2buffer4xi = create2buffer4vpu()
            message2print = "{(Step 1H/70) set" + "; vpu2buffer4xi-set-by-copy2imagetobuffer4vpu to: " + str(vpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4vpu',
            #   'METHOD': 'create2buffer4vpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            vpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; vpu2buffer4xi-yielded-by-copy2imagetobuffer4vpu: " + str(vpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2vpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2vpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2vpu-created-by-copy2imagetobuffer4vpu to: " + str(task4copyimagetobuffer2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4vpu = trace2image4vpu                                                                 # CALLBACK FUNCTION FOR VPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-copy2imagetobuffer4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4vpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4vpu"                                                            # MODULE OF VPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4vpu"                                                            # FUNCTION OF VPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4vpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4vpu', 
        #  'METHOD': 'copy2imagetobuffer4vpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2vpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2vpu-created-by-copy2imagetobuffer4vpu to: " + str(task4copyimagetobuffer2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4vpu', 
        #  'METHOD': 'copy2imagetobuffer4vpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2vpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2vpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-copy2imagetobuffer4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-copy2imagetobuffer4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4vpu', 
            #  'METHOD': 'create2context4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-copy2imagetobuffer4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2vpu ['CONTEXT'] = context2vpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2vpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-copy2imagetobuffer4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-copy2imagetobuffer4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_VPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-copy2imagetobuffer4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2vpu ['DEVICE'] = device2vpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2vpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(imagesourceinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2vpu = get2imageinfo4vpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(imagesourceinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(imagesourceinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2vpu ['IMAGESOURCE'] = imagesourceinfo2vpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2vpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(bufferdestinationinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2vpu = get2bufferinfo4vpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(bufferdestinationinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4vpu', 
            #  'METHOD': 'create2buffer4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(bufferdestinationinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2vpu ['BUFFERDESTINATION'] = bufferdestinationinfo2vpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY VPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4vpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4vpu-trying-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4vpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4vpu-got-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4vpu-failed-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2vpu ['QUEUE'] = queue2copyimagetobuffer4vpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY VPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2vpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2vpu-trying-by-copy2imagetobuffer4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2vpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2vpu-got-by-copy2imagetobuffer4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2vpu-failed-by-copy2imagetobuffer4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2vpu ['PIPE'] = pipe2vpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2vpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4vpu', 
            #  'METHOD': 'create2commandqueue4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2vpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2vpu = get2eventinfo4vpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4vpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4vpu', 
            #  'METHOD': 'create2event4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4vpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2vpu ['EVENT'] = eventinfo2vpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2vpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2vpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2vpu-set-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2vpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2vpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2vpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2vpu-trying-by-copy2imagetobuffer4vpu: " + str(origin4source2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2vpu['X'] = 0 
            origin4source2vpu['Y'] = 0  
            origin4source2vpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2vpu-set-by-copy2imagetobuffer4vpu to: " + str(origin4source2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2vpu-failed-by-copy2imagetobuffer4vpu: " + str(origin4source2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2vpu ['SOURCEORIGIN'] = origin4source2vpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2vpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2vpu-trying-by-copy2imagetobuffer4vpu: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2vpu['X'] = 10 
            region4copy2vpu['Y'] = 10  
            region4copy2vpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2vpu-set-by-copy2imagetobuffer4vpu to: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2vpu-failed-by-copy2imagetobuffer4vpu: " + str(region4copy2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2vpu ['REGION'] = region4copy2vpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2vpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2vpu-set-by-copy2imagetobuffer4vpu to: " + str(offset4copy2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2vpu ['OFFSET'] = offset4copy2vpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2vpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4vpu = trace2image4vpu                                                             # CALLBACK FUNCTION FOR VPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-copy2imagetobuffer4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4vpu"                                                        # MODULE OF VPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4vpu"                                                        # FUNCTION OF VPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2vpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2vpu-created-by-copy2imagetobuffer4vpu to: " + str(task4copyimagetobuffer2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2vpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2vpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-copy2imagetobuffer4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2vpu = get2context4vpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-copy2imagetobuffer4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-copy2imagetobuffer4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2vpu ['CONTEXT'] = context2vpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2vpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-copy2imagetobuffer4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2vpu = get2vpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-copy2imagetobuffer4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-copy2imagetobuffer4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2vpu ['DEVICE'] = device2vpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2vpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(sourceimageinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2vpu = get2imageinfo4vpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(sourceimageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(sourceimageinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2vpu ['IMAGESOURCE'] = sourceimageinfo2vpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2vpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(destinationbufferinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2vpu = get2bufferinfo4vpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(destinationbufferinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(destinationbufferinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2vpu ['BUFFERDESTINATION'] = destinationbufferinfo2vpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY VPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4vpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4vpu-trying-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4vpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4vpu-got-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4vpu-failed-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2vpu ['QUEUE'] = queue2copyimagetobuffer4vpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY VPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2vpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2vpu-trying-by-copy2imagetobuffer4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2vpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2vpu-got-by-copy2imagetobuffer4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2vpu-failed-by-copy2imagetobuffer4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2vpu ['PIPE'] = pipe2vpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2vpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2vpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2vpu = get2eventinfo4vpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2vpu-got-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2vpu ['EVENT'] = eventinfo2vpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2vpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2vpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2vpu-set-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2vpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2vpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2imagetobuffer4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2vpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2vpu-trying-by-copy2imagetobuffer4vpu: " + str(source4origin2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2vpu['X'] = 0 
            source4origin2vpu['Y'] = 0  
            source4origin2vpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2vpu-set-by-copy2imagetobuffer4vpu to: " + str(source4origin2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2vpu-failed-by-copy2imagetobuffer4vpu: " + str(source4origin2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2vpu ['SOURCEORIGIN'] = source4origin2vpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2vpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2vpu-trying-by-copy2imagetobuffer4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2vpu-set-by-copy2imagetobuffer4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2vpu-failed-by-copy2imagetobuffer4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2vpu ['REGION'] = region2vpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2vpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2vpu-set-by-copy2imagetobuffer4vpu to: " + str(offset2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2vpu ['OFFSET'] = offset2vpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2vpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-copy2imagetobuffer4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4vpu, publisherchannel2copyimagetobuffer4vpu = task4copyimagetobuffer2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4vpu-got-by-copy2imagetobuffer4vpu: " + str(consumerchannel2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4vpu-got-by-copy2imagetobuffer4vpu: " + str(publisherchannel2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2vpu = True
        
        control2copyimagetobuffer4vpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4vpu
        
        
        control2copyimagetobuffer4vpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4vpu = False
    queue2copyimagetobuffer4vpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4vpu-trying-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4vpu = task4copyimagetobuffer2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4vpu-got-by-copy2imagetobuffer4vpu: " + str(queue2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4vpu = True
        control2copyimagetobuffer4vpu['QUEUE']  = queue2copyimagetobuffer4vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-copy2imagetobuffer4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4vpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4vpu, publisherchannel2copyimagetobuffer4vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-copy2imagetobuffer4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4vpu-returning-by-copy2imagetobuffer4vpu: " + str(control2copyimagetobuffer4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4vpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4vpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4VPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4VPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4vpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4VPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4vpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4vpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4vpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4VPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4vpu = "<copy4xi>copy2copybuffertoimage4vpu</copy4xi>"
#       channel2copybuffertoimagepublish4vpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4vpu.send(message2copybuffertoimagesend4vpu)
#       print("(Step 81B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copybuffertoimagepublish4vpu, " MESSAGE2SEND4VPU: " , message2copybuffertoimagesend4vpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copybuffertoimage4vpu(queue2copybuffertoimage4vpu, channel2copybuffertoimageconsume4vpu, channel2copybuffertoimagepublish4vpu)
#       print("(Step 82D/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copybuffertoimage4vpu)
#   except:
#       print("(Step 82Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4vpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copybuffertoimageconsume4vpu.recv()
#       print("(Step 83B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copybuffertoimageconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4VPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4vpu = "<copy4xi>copy2copybuffertoimage4vpu</copy4xi>"
#       channel2copybuffertoimagepublish4vpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4vpu.send(message2copybuffertoimagesend4vpu)
#       print("(Step 84B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2copybuffertoimagepublish4vpu, " MESSAGE2SEND4VPU: " , message2copybuffertoimagesend4vpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copybuffertoimage4vpu(queue2copybuffertoimage4vpu, channel2copybuffertoimageconsume4vpu, channel2copybuffertoimagepublish4vpu)
#       print("(Step 85B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copybuffertoimage4vpu)
#   except:
#       print("(Step 85Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4VPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4vpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4vpu = channel2copybuffertoimageconsume4vpu.recv()
#       print("(Step 86B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copybuffertoimageconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4VPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copybuffertoimage4vpu(queue2copybuffertoimage4vpu, channel2copybuffertoimageconsume4vpu, channel2copybuffertoimagepublish4vpu)
#       print("(Step 88B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", manoeuvre2copybuffertoimage4vpu)
#   except:
#       print("(Step 88Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4vpu = channel2copybuffertoimageconsume4vpu.recv()
#       print("(Step 89D/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2copybuffertoimageconsume4vpu, " CONTROL & COMMAND: ", message2copyimagereceive4vpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4VPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4vpu = manoeuvre2copybuffertoimage4vpu(queue2copybuffertoimage4vpu, channel2copybuffertoimageconsume4vpu, channel2copybuffertoimagepublish4vpu)
#       print("(Step 90B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvreimagecopytobuffer4vpu)
#   except:
#       print("(Step 90Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4VPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4vpu(task4copybuffertoimage2vpu=None):

    '''
    copies buffer to image for VPU
    '''
    global vpu2image4xi                                                                                             # 2D-IMAGES FOR VPU  
    global vpu2buffer4xi                                                                                            # BUFFER FOR VPU
    
    global control2copybuffertoimage4vpu                                                                            # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4vpu"
    method2print = "copy2buffertoimage4vpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4vpu-started: " + str(copy2buffertoimage4vpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4vpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2vpu-to-copy2buffertoimage4vpu: " + str(task4copybuffertoimage2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) created" + "; vpu2image4xi-created-by-copy2buffertoimage4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-copy2buffertoimage4vpu: " + str(copy2buffertoimage4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-copy2buffertoimage4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; vpu2image4xi-yielded-by-copy2buffertoimage4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2buffer4xi is None:
            vpu2buffer4xi = create2buffer4vpu()                                                                     # INITIALIZE BUFFER FOR VPU 
            message2print = "{(Step 1F/70) created" + "; vpu2buffer4xi-created-by-copy2buffertoimage4vpu to: " + str(vpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4vpu-skipped-by-copy2buffertoimage4vpu: " + str(copy2buffertoimage4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            vpu2buffer4xi = create2buffer4vpu()
            message2print = "{(Step 1H/70) set" + "; vpu2buffer4xi-set-by-copy2buffertoimage4vpu to: " + str(vpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; vpu2buffer4xi-yielded-by-copy2buffertoimage4vpu: " + str(vpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2vpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2vpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2vpu-created-by-copy2buffertoimage4vpu to: " + str(task4copybuffertoimage2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4vpu = trace2image4vpu                                                                 # CALLBACK FUNCTION FOR VPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-copy2buffertoimage4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4vpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4vpu"                                                     # MODULE OF VPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4vpu"                                                     # FUNCTION OF VPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4vpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4vpu', 
        #  'METHOD': 'copy2buffertoimage4vpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2vpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2vpu-created-by-copy2buffertoimage4vpu to: " + str(task4copybuffertoimage2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4vpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4vpu', 
        #  'METHOD': 'copy2buffertoimage4vpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2vpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2vpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-copy2buffertoimage4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-copy2buffertoimage4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-copy2buffertoimage4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2vpu ['CONTEXT'] = context2vpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2vpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-copy2buffertoimage4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-copy2buffertoimage4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-copy2buffertoimage4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2vpu ['DEVICE'] = device2vpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2vpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(imagedestinationinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2vpu = get2imageinfo4vpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(imagedestinationinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4vpu', 
            #  'METHOD': 'create2context4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4vpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(imagedestinationinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2vpu ['IMAGEDESTINATION'] = imagedestinationinfo2vpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2vpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(buffersourceinfo2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2vpu = get2bufferinfo4vpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(buffersourceinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4vpu', 
            #  'METHOD': 'create2buffer4vpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(buffersourceinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2vpu ['BUFFERSOURCE'] = buffersourceinfo2vpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY VPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4vpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4vpu-trying-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4vpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4vpu-got-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4vpu-failed-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2vpu ['QUEUE'] = queue2copybuffertoimage4vpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY VPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2vpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2vpu-trying-by-copy2buffertoimage4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2vpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2vpu-got-by-copy2buffertoimage4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2vpu-failed-by-copy2buffertoimage4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2vpu ['PIPE'] = pipe2vpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2vpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2vpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2vpu = get2eventinfo4vpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2vpu ['EVENT'] = eventinfo2vpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2vpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2vpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2vpu-set-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2vpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2vpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2vpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2vpu-trying-by-copy2buffertoimage4vpu: " + str(origin4destination2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2vpu['X'] = 100 
            origin4destination2vpu['Y'] = 100  
            origin4destination2vpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2vpu-set-by-copy2buffertoimage4vpu to: " + str(origin4destination2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2vpu-failed-by-copy2buffertoimage4vpu: " + str(origin4destination2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2vpu ['DESTINATIONORIGIN'] = origin4destination2vpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2vpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2vpu-trying-by-copy2buffertoimage4vpu: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2vpu['X'] = 10 
            region4copy2vpu['Y'] = 10  
            region4copy2vpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2vpu-set-by-copy2buffertoimage4vpu to: " + str(region4copy2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2vpu-failed-by-copy2buffertoimage4vpu: " + str(region4copy2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2vpu ['REGION'] = region4copy2vpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2vpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2vpu-set-by-copy2buffertoimage4vpu to: " + str(offset4copy2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2vpu ['OFFSET'] = offset4copy2vpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2vpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                             # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-copy2buffertoimage4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4vpu"                                                        # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4vpu"                                                        # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2vpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2vpu-created-by-copy2buffertoimage4vpu to: " + str(task4copybuffertoimage2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2vpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-copy2buffertoimage4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-copy2buffertoimage4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-copy2buffertoimage4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2vpu ['CONTEXT'] = context2vpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-copy2buffertoimage4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-copy2buffertoimage4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-copy2buffertoimage4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2vpu ['DEVICE'] = device2vpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2vpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(destinationimageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(destinationimageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(destinationimageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2vpu ['IMAGEDESTINATION'] = destinationimageinfo2vpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2vpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(sourcebufferinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2vpu = get2bufferinfo4vpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(sourcebufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(sourcebufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2vpu ['BUFFERSOURCE'] = sourcebufferinfo2vpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4vpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4vpu-trying-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4vpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4vpu-got-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4vpu-failed-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2vpu ['QUEUE'] = queue2copybuffertoimage4vpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2vpu-trying-by-copy2buffertoimage4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2vpu-got-by-copy2buffertoimage4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2vpu-failed-by-copy2buffertoimage4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2vpu ['PIPE'] = pipe2vpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2vpu-got-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2vpu ['EVENT'] = eventinfo2vpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2vpu-set-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2vpu-set-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-copy2buffertoimage4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2vpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2vpu-trying-by-copy2buffertoimage4vpu: " + str(destination4origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2vpu['X'] = 0 
                destination4origin2vpu['Y'] = 0  
                destination4origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2vpu-set-by-copy2buffertoimage4vpu to: " + str(destination4origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2vpu-failed-by-copy2buffertoimage4vpu: " + str(destination4origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2vpu ['DESTINATIONORIGIN'] = destination4origin2vpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2vpu-trying-by-copy2buffertoimage4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2vpu-set-by-copy2buffertoimage4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2vpu-failed-by-copy2buffertoimage4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2vpu ['REGION'] = region2vpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2vpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2vpu-set-by-copy2buffertoimage4vpu to: " + str(offset2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2vpu ['OFFSET'] = offset2vpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2vpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-copy2buffertoimage4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4vpu, publisherchannel2copybuffertoimage4vpu = task4copybuffertoimage2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4vpu-got-by-copy2buffertoimage4vpu: " + str(consumerchannel2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4vpu-got-by-copy2buffertoimage4vpu: " + str(publisherchannel2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2vpu = True
        
        control2copybuffertoimage4vpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4vpu
        
        
        control2copybuffertoimage4vpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4vpu = False
    queue2copybuffertoimage4vpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4vpu-trying-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4vpu = task4copybuffertoimage2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4vpu-got-by-copy2buffertoimage4vpu: " + str(queue2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4vpu = True
        control2copybuffertoimage4vpu['QUEUE']  = queue2copybuffertoimage4vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-copy2buffertoimage4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4vpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4vpu, publisherchannel2copybuffertoimage4vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-copy2buffertoimage4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4vpu-returning-by-copy2buffertoimage4vpu: " + str(control2copybuffertoimage4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4vpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4vpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4vpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4VPU : ', response4read2image)
#       queue2read4vpu      = response4read2image['QUEUE']
#       channel2readpublish4vpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4vpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2readsend4vpu = "<copy4xi>copy2read4vpu</copy4xi>"
#       channel2readpublish4vpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4vpu.send(message2readsend4vpu)
#       print("(Step 17B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2readpublish4vpu, " MESSAGE2SEND4VPU: " , message2readsend4vpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4VPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2read4vpu, channel2readconsume4vpu, channel2readpublish4vpu)
#       print("(Step 18D/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 18Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:
#       channel2readconsume4vpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4vpu = channel2readconsume4vpu.recv()
#       print("(Step 19B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2readconsume4vpu, " CONTROL & COMMAND: ", message2readreceive4vpu)                                                                                                  
#       # <roger4xi>roger2read4vpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4VPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2readsend4vpu = "<ready4xi>ready2read4vpu</ready4xi>"
#       channel2readpublish4vpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4vpu.send(message2readsend4vpu)
#       print("(Step 20B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2readpublish4vpu, " MESSAGE2SEND4VPU: " , message2readsend4vpu)                                                                                                
#       # <ready4xi>ready2read4vpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4VPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2read4vpu, channel2readconsume4vpu, channel2readpublish4vpu)
#       print("(Step 21B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 21Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:
#       channel2readconsume4vpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4vpu = channel2readconsume4vpu.recv()
#       print("(Step 22B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2readconsume4vpu, " CONTROL & COMMAND: ", message2readreceive4vpu)                                                                                                  
#       # <ready4xi>ready2read4vpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4VPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4VPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2read4vpu, channel2readconsume4vpu, channel2readpublish4vpu)
#       print("(Step 25B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 25Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4vpu = channel2readconsume4vpu.recv()
#       print("(Step 26D/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2readconsume4vpu, " CONTROL & COMMAND: ", message2readreceive4vpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4vpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4VPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2read4vpu, channel2readconsume4vpu, channel2readpublish4vpu)
#       print("(Step 27B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvre4vpu)
#   except:
#       print("(Step 27Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4vpu(task4imageread2vpu=None):

    '''
    reads 2D images for VPU
    '''
    global vpu2image4xi                                                     # 2D-IMAGES FOR VPU  
    global control2readimage4vpu                                              # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4vpu"
    method2print = "read2image4vpu"

    message2print = "{(Step 1A/33) started" + "; read2image4vpu-started: " + str(read2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4vpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2vpu-to-read2image4vpu: " + str(task4imageread2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) created" + "; vpu2image4xi-created-by-read2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-read2image4vpu: " + str(read2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-read2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; vpu2image4xi-yielded-by-read2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2vpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2vpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2vpu-created-by-read2image4vpu to: " + str(task4imageread2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                              # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-read2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4vpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4vpu"                                                          # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4vpu"                                                          # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4vpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4vpu', 
            #  'METHOD': 'read2image4vpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2vpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2vpu-created-by-read2image4vpu to: " + str(task4imageread2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4vpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4vpu', 
            #  'METHOD': 'read2image4vpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2vpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-read2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-read2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-read2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2vpu ['CONTEXT'] = context2vpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-read2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-read2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-read2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2vpu ['DEVICE'] = device2vpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2vpu-trying-by-read2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2vpu-got-by-read2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2vpu-failed-by-read2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2vpu ['IMAGE'] = imageinfo2vpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2vpu-trying-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2vpu-got-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2vpu-failed-by-read2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2vpu ['QUEUE'] = queue2vpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2vpu-trying-by-read2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2vpu-got-by-read2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2vpu-failed-by-read2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2vpu ['PIPE'] = pipe2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2vpu-trying-by-read2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2vpu-got-by-read2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2vpu-failed-by-read2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2vpu-trying-by-read2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                                   # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2vpu-got-by-read2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2vpu-failed-by-read2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2vpu ['BUFFER'] = bufferinfo2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2vpu-trying-by-read2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2vpu-got-by-read2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2vpu-failed-by-read2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2vpu ['EVENT'] = eventinfo2vpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2vpu-trying-by-read2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2vpu-set-by-read2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2vpu-failed-by-read2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-read2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2vpu-set-by-read2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-read2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2vpu-trying-by-read2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2vpu-set-by-read2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2vpu-failed-by-read2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2vpu ['BLOCKING'] = blocking2vpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2vpu-trying-by-read2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2vpu-set-by-read2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2vpu-failed-by-read2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2vpu ['ORIGIN'] = origin2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2vpu-trying-by-read2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2vpu-set-by-read2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2vpu-failed-by-read2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2vpu ['REGION'] = region2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2vpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2vpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2vpu-trying-by-read2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2vpu-set-by-read2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2vpu-failed-by-read2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2vpu ['ROWPITCH'] = rowpitch2vpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2vpu = None
            height2vpu = 800
            size2vpu = int(width2vpu * height2vpu)                                                                  # CALCULATED MEMORY
            if size2vpu == 0:
                size2vpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2vpu-trying-by-read2image4vpu: " + str(pointer2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2vpu = Arena4Xi(size2vpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2vpu-got-by-read2image4vpu: " + str(arena2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2vpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4vpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2vpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4vpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2vpu:
                    size2vpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2vpu-set-by-read2buffer4vpu to: " + str(size2vpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2vpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2vpu-set-by-read2buffer4vpu to: " + str(pointer2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2vpu-failed-by-read2image4vpu: " + str(pointer2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2vpu ['POINTER'] = pointer2vpu                            
            task4imageread2vpu ['SIZE']    = size2vpu                                                               # AUTOMATIC BY VPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2vpu-trying-by-read2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2vpu-set-by-read2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2vpu-failed-by-read2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2vpu ['SLICEPITCH'] = slicepitch2vpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2vpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                              # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-read2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4vpu"                                                          # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4vpu"                                                          # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2vpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2vpu-created-by-read2image4vpu to: " + str(task4imageread2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2vpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-read2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-read2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-read2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2vpu ['CONTEXT'] = context2vpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-read2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-read2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-read2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2vpu ['DEVICE'] = device2vpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2vpu-trying-by-read2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2vpu-got-by-read2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2vpu-failed-by-read2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2vpu ['IMAGE'] = imageinfo2vpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2vpu-trying-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2vpu-got-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2vpu-failed-by-read2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2vpu ['QUEUE'] = queue2vpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2vpu-trying-by-read2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2vpu-got-by-read2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2vpu-failed-by-read2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2vpu ['PIPE'] = pipe2vpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2vpu-trying-by-read2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2vpu-got-by-read2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2vpu-failed-by-read2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2vpu-trying-by-read2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                                   # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2vpu-got-by-read2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2vpu-failed-by-read2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2vpu ['BUFFER'] = bufferinfo2vpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2vpu-trying-by-read2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2vpu-got-by-read2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2vpu-failed-by-read2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2vpu ['EVENT'] = eventinfo2vpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2vpu-trying-by-read2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2vpu-set-by-read2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2vpu-failed-by-read2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-read2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2vpu-set-by-read2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-read2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2vpu-trying-by-read2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2vpu-set-by-read2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2vpu-failed-by-read2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2vpu ['BLOCKING'] = blocking2vpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2vpu-trying-by-read2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2vpu-set-by-read2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2vpu-failed-by-read2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2vpu ['ORIGIN'] = origin2vpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2vpu-trying-by-read2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2vpu-set-by-read2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2vpu-failed-by-read2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2vpu ['REGION'] = region2vpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2vpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2vpu-trying-by-read2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2vpu-set-by-read2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2vpu-failed-by-read2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2vpu ['ROWPITCH'] = rowpitch2vpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2vpu = None
            size2vpu = int(width2vpu * height2vpu)                                                                  # CALCULATED MEMORY
            if size2vpu == 0:
                size2vpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2vpu-trying-by-read2image4vpu: " + str(pointer2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2vpu = Arena4Xi(size2vpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2vpu-got-by-read2image4vpu: " + str(arena2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2vpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4vpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2vpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4vpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2vpu:
                    size2vpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2vpu-set-by-read2buffer4vpu to: " + str(size2vpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2vpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2vpu-set-by-read2buffer4vpu to: " + str(pointer2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2vpu-failed-by-read2image4vpu: " + str(pointer2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2vpu ['POINTER'] = pointer2vpu                            
            task4imageread2vpu ['SIZE']    = size2vpu                                                               # AUTOMATIC BY VPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2vpu-trying-by-read2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2vpu-set-by-read2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2vpu-failed-by-read2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2vpu ['SLICEPITCH'] = slicepitch2vpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2vpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2vpu-yielded-by-read2image4vpu: " + str(task4imageread2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-read2image4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2vpu, publisherchannel2vpu = task4imageread2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2vpu-got-by-read2image4vpu: " + str(consumerchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2vpu-got-by-read2image4vpu: " + str(publisherchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2vpu = True
        
        control2readimage4vpu['CONSUMERCHANNEL']  = consumerchannel2vpu
        
        
        control2readimage4vpu['PUBLISHERCHANNEL'] = publisherchannel2vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2vpu = False
    queue2vpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2vpu-trying-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2vpu = task4imageread2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2vpu-got-by-read2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2vpu = True
        control2readimage4vpu['QUEUE']  = queue2vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-read2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2readimage4vpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2vpu, publisherchannel2vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-read2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4vpu-returning-by-read2image4vpu: " + str(control2readimage4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4vpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4vpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4VPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4vpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4VPU : ', response4map2image)
#    
#       queue4map2image4vpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4vpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4vpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4VPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4VPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())      
#   try:                               
#       message2send4vpu = "<copy4xi>copy2mapimage4vpu</copy4xi>"
#       channel4map2imagepublish4vpu.send(message2send4vpu)
#       print("(Step 93B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel4map2imagepublish4vpu, " MESSAGE2SEND4VPU: " , message2send4vpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4VPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 94A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4vpu(queue4map2image4vpu, channel4map2imageconsume4vpu, channel4map2imagepublish4vpu)
#           print("(Step 94B/700) OF VPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4VPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())      
#   try:
#       message2receive4vpu = channel4map2imageconsume4vpu.recv()
#       print("(Step 95B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel4map2imageconsume4vpu, " CONTROL & COMMAND: ", message2receive4vpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4VPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4VPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2send4vpu = "<ready4xi>ready2mapimage4vpu</ready4xi>"
#       channel4map2imagepublish4vpu.send(message2send4vpu)
#       print("(Step 96B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel4map2imagepublish4vpu, " MESSAGE2SEND4VPU: " , message2send4vpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4VPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 97A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4vpu(queue4map2image4vpu, channel4map2imageconsume4vpu, channel4map2imagepublish4vpu)
#           print("(Step 97B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", manoeuvre2copyimage4vpu)
#       except:
#           print("(Step 97Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4VPU ', time4xi())     
#   try:
#       message2receive4vpu = channel4map2imageconsume4vpu.recv()
#       print("(Step 98B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel4map2imageconsume4vpu, " CONTROL & COMMAND: ", message2receive4vpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4VPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE VPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF VPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE VPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF VPU2IMAGE4XI5TEST - FILENO: ", file2vpu.fileno( ))  
#       print("(Step 99C/700) OF VPU2IMAGE4XI5TEST - MODE: ", file2vpu.mode)
#       print("(Step 99D/700) OF VPU2IMAGE4XI5TEST - NAME: ", file2vpu.name)
#       offset2vpu = 10
#       file2vpu.seek(offset2vpu)
#       print("(Step 99E/700) OF VPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2vpu)
#       file2vpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4VPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 101A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4vpu(queue4map2image4vpu, channel4map2imageconsume4vpu, channel4map2imagepublish4vpu)
#           print("(Step 101B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", manoeuvre2copyimage4vpu)
#       except:
#           print("(Step 101Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4vpu = channel4map2imageconsume4vpu.recv()
#       print("(Step 102B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel4map2imageconsume4vpu, " CONTROL & COMMAND: ", message2receive4vpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4VPU  - LET MANOEUVRE2COPYIMAGE4VPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 103A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4vpu(queue4map2image4vpu, channel4map2imageconsume4vpu, channel4map2imagepublish4vpu)
#           print("(Step 103B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4vpu(task4imagemap2vpu=None):

    '''
    maps 2D images for VPU
    '''
    global vpu2image4xi                                                                                             # 2D-IMAGES FOR VPU  
    global control2mapimage4vpu                                                                                     # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4vpu"
    method2print = "map2image4vpu"

    message2print = "{(Step 1A/33) started" + "; map2image4vpu-started: " + str(map2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2vpu-to-map2image4vpu: " + str(task4imagemap2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) created" + "; vpu2image4xi-created-by-map2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-map2image4vpu: " + str(map2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-map2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; vpu2image4xi-yielded-by-map2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2vpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2vpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2vpu-created-by-map2image4vpu to: " + str(task4imagemap2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                              # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-map2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4vpu"                                                          # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4vpu"                                                          # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2vpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2vpu-created-by-map2image4vpu to: " + str(task4imagemap2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2vpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-map2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-map2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-map2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2vpu ['CONTEXT'] = context2vpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-map2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-map2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-map2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2vpu ['DEVICE'] = device2vpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2vpu-trying-by-map2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2vpu-got-by-map2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2vpu-failed-by-map2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2vpu ['IMAGE'] = imageinfo2vpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2vpu-trying-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2vpu-got-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2vpu-failed-by-map2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2vpu ['QUEUE'] = queue2vpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2vpu-trying-by-map2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2vpu-got-by-map2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2vpu-failed-by-map2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2vpu ['PIPE'] = pipe2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2vpu-trying-by-map2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2vpu-got-by-map2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2vpu-failed-by-map2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2vpu-trying-by-map2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                                   # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2vpu-got-by-map2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2vpu-failed-by-map2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2vpu ['BUFFER'] = bufferinfo2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2vpu-trying-by-map2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2vpu-got-by-map2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2vpu-failed-by-map2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2vpu ['EVENT'] = eventinfo2vpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2vpu-trying-by-map2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2vpu-set-by-map2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2vpu-failed-by-map2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-map2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2vpu-set-by-map2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-map2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2vpu-trying-by-map2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2vpu-set-by-map2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2vpu-failed-by-map2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2vpu ['BLOCKING'] = blocking2vpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2vpu-trying-by-map2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2vpu-set-by-map2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2vpu-failed-by-map2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2vpu ['ORIGIN'] = origin2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2vpu-trying-by-map2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2vpu-set-by-map2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2vpu-failed-by-map2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2vpu ['REGION'] = region2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2vpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2vpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2vpu-trying-by-map2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2vpu-set-by-map2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2vpu-failed-by-map2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2vpu ['ROWPITCH'] = rowpitch2vpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2vpu-trying-by-map2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2vpu-set-by-map2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2vpu-failed-by-map2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2vpu ['SLICEPITCH'] = slicepitch2vpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2vpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2vpu-trying-by-map2image4vpu: " + str(flags2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2vpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2vpu-set-by-map2image4vpu to: " + str(flags2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2vpu-failed-by-map2image4vpu: " + str(flags2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2vpu ['FLAGS'] = flags2vpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2vpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                              # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-map2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4vpu"                                                          # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4vpu"                                                          # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2vpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2vpu-created-by-map2image4vpu to: " + str(task4imagemap2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2vpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-map2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-map2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-map2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2vpu ['CONTEXT'] = context2vpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-map2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-map2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-map2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2vpu ['DEVICE'] = device2vpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2vpu-trying-by-map2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2vpu-got-by-map2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2vpu-failed-by-map2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2vpu ['IMAGE'] = imageinfo2vpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2vpu-trying-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2vpu-got-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2vpu-failed-by-map2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2vpu ['QUEUE'] = queue2vpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2vpu-trying-by-map2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2vpu-got-by-map2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2vpu-failed-by-map2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2vpu ['PIPE'] = pipe2vpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2vpu-trying-by-map2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2vpu-got-by-map2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2vpu-failed-by-map2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2vpu-trying-by-map2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                                   # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2vpu-got-by-map2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2vpu-failed-by-map2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2vpu ['BUFFER'] = bufferinfo2vpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2vpu-trying-by-map2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2vpu-got-by-map2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2vpu-failed-by-map2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2vpu ['EVENT'] = eventinfo2vpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2vpu-trying-by-map2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2vpu-set-by-map2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2vpu-failed-by-map2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-map2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2vpu-set-by-map2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-map2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2vpu-trying-by-map2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2vpu-set-by-map2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2vpu-failed-by-map2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2vpu ['BLOCKING'] = blocking2vpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2vpu-trying-by-map2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2vpu-set-by-map2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2vpu-failed-by-map2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2vpu ['ORIGIN'] = origin2vpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2vpu-trying-by-map2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2vpu-set-by-map2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2vpu-failed-by-map2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2vpu ['REGION'] = region2vpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2vpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2vpu-trying-by-map2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2vpu-set-by-map2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2vpu-failed-by-map2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2vpu ['ROWPITCH'] = rowpitch2vpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2vpu-trying-by-map2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2vpu-set-by-map2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2vpu-failed-by-map2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2vpu ['SLICEPITCH'] = slicepitch2vpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2vpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2vpu-trying-by-map2image4vpu: " + str(flags2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2vpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2vpu-set-by-map2image4vpu to: " + str(flags2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2vpu-failed-by-map2image4vpu: " + str(flags2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2vpu ['FLAGS'] = flags2vpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2vpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2vpu-yielded-by-map2image4vpu: " + str(task4imagemap2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-map2image4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2vpu, publisherchannel2vpu = task4imagemap2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2vpu-got-by-map2image4vpu: " + str(consumerchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2vpu-got-by-map2image4vpu: " + str(publisherchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2vpu = True
        
        control2mapimage4vpu['CONSUMERCHANNEL']  = consumerchannel2vpu
        
        
        control2mapimage4vpu['PUBLISHERCHANNEL'] = publisherchannel2vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2vpu = False
    queue2vpu = None
    message2print = "{(Step 41A/70) trying" + "queue2vpu-trying-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2vpu = task4imagemap2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2vpu-got-by-map2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2vpu = True
        control2mapimage4vpu['QUEUE']  = queue2vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-map2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2mapimage4vpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2vpu, publisherchannel2vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-map2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4vpu-returning-by-map2image4vpu: " + str(control2mapimage4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4vpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4vpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4VPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 42A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4vpu(queue2copy4vpu, channel2copyconsume4vpu, channel2copypublish4vpu)
#           print("(Step 42B/700) OF VPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4VPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4VPU': '<roger4xi>roger2copy4vpu</roger4xi>', 'ACK2INFO4VPU': False, 'MESSAGE2GET4VPU': '<default4xi>default2copy4vpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copy4vpu</copy4xi>', 
#           'CODE2ACK4VPU': ' 0D-1C-', 'MESSAGE2PUT4VPU': '<default4xi>default2copy4vpu</default4xi>', 'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 1}
#        except:
#           print("(Step 42Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4vpu(queue2copymanoeuvre4vpu, consumerchannel2copymanoeuvre4vpu, publisherchannel2copymanoeuvre4vpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4vpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4vpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4vpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4vpu                                 
    global message2copyimageput4vpu
    global message2copyimagereceive4vpu
    global message2copyimagesend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4vpu" 
    method2print = "manoeuvre2copyimage4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4vpu-started: " + str(manoeuvre2copyimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4vpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4vpu-to-manoeuvre2copyimage4vpu: " + str(queue2copymanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4vpu-to-manoeuvre2copyimage4vpu: " + str(consumerchannel2copymanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4vpu-to-manoeuvre2copyimage4vpu: " + str(publisherchannel2copymanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4vpu-try-by-manoeuvre2copyimage4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4vpu is None:
            state2copyimage4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(state2copyimage4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4vpu-used-by-manoeuvre2copyimage4vpu: " + str(state2copyimage4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(state2copyimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(ack2copyimageinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4vpu     = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(message2copyimageget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimageput4vpu     = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(message2copyimageput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimagereceive4vpu = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(message2copyimagereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimagesend4vpu    = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        max2copyimagestate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4vpu-initialized-by-manoeuvre2copyimage4vpu to: " + str( max2copyimagestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4vpu = state2copyimage4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4vpu-added-by-manoeuvre2copyimage4vpu: " + str(state2copyimage4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4vpu > max2copyimagestate4vpu:
        state2copyimage4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(state2copyimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(ack2copyimageinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4vpu     = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(message2copyimageget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimageput4vpu     = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(message2copyimageput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimagereceive4vpu = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(message2copyimagereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        message2copyimagesend4vpu    = '<default4xi>default2copy4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4vpu</default4xi>
        max2copyimagestate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4vpu-set-by-manoeuvre2copyimage4vpu to: " + str( max2copyimagestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4vpu = consumerchannel2copymanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4vpu-received-by-manoeuvre2copyimage4vpu: " + str(message2copyimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4vpu</copy4xi>
                         
            if message2copyimagereceive4vpu == '<copy4xi>copy2copy4vpu</copy4xi>':
                message2copyimagesend4vpu = '<roger4xi>roger2copy4vpu</roger4xi>'
            else:
                message2copyimagesend4vpu = '<negative4xi><negative2copy4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4vpu.send(message2copyimagesend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4vpu-sent-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4vpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(message2copyimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4vpu = consumerchannel2copymanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4vpu-received-by-manoeuvre2copyimage4vpu: " + str(message2copyimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4vpu</ready4xi>
                         
            if message2copyimagereceive4vpu == '<ready4xi>ready2copy4vpu</ready4xi>':
                message2copyimagesend4vpu = '<ready4xi>ready2copy4vpu</ready4xi>'
            else:
                message2copyimagesend4vpu = '<negative4xi><negative2copy4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4vpu.send(message2copyimagesend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4vpu-sent-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(message2copyimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2copyimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2copymanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2copyimage4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(queue2copymanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4vpu is True:
                message2copyimagesend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2copyimagesend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4vpu-set-by-manoeuvre2copyimage4vpu to: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4vpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4vpu-calling-by-manoeuvre2copyimage4vpu: " + str(publisherchannel2copymanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4vpu-passing-to-send-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4vpu.send(message2copyimagesend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4vpu-sent-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4vpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4vpu-failed-by-manoeuvre2copyimage4vpu: " + str(message2copyimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4VPU']        = ack2copyimageinfo4vpu
    response2manouvre4copy['CODE2ACK4VPU']        = code2ack
    response2manouvre4copy['CODE2NACK4VPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4VPU']     = message2copyimageget4vpu
    response2manouvre4copy['MESSAGE2PUT4VPU']     = message2copyimageput4vpu
    response2manouvre4copy['MESSAGE2RECEIVE4VPU'] = message2copyimagereceive4vpu
    response2manouvre4copy['MESSAGE2SEND4VPU']    = message2copyimagesend4vpu
    response2manouvre4copy['STATE2MANOEUVRE4VPU'] = state2copyimage4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4vpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4VPU': '<roger4xi>roger2copy4vpu</roger4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copy4vpu</copy4xi>',
    #   'CODE2ACK4VPU': ' 0D-1C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4VPU': '<ready4xi>ready2copy4vpu</ready4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copy4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-2C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>',
    #   'ACK2INFO4VPU': True,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copy4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'CODE2ACK4VPU': ' 0D-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copy4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4vpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4VPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 62A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4vpu(queue4copy2imagetobuffer4vpu, channel4copy2imagetobufferconsume4vpu, channel4copy2imagetobufferpublish4vpu)
#           print("(Step 62B/700) OF VPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4VPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4VPU': '<roger4xi>roger2copyimagetobuffer4vpu</roger4xi>',
#           #   'ACK2INFO4VPU': False,
#           #   'MESSAGE2GET4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
#           #   'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copyimagetobuffer4vpu</copy4xi>',
#           #   'CODE2ACK4VPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
#           #   'CODE2NACK4VPU': ' ',
#           #   'STATE2MANOEUVRE4VPU': 1}
#        except:
#           print("(Step 62Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4vpu(queue2copyimagetobuffermanoeuvre4vpu, consumerchannel2copyimagetobuffermanoeuvre4vpu, publisherchannel2copyimagetobuffermanoeuvre4vpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4vpu
    '''
    global state2copyimagetobuffer4vpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4vpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4vpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4vpu                                 
    global message2copyimagetobufferput4vpu
    global message2copyimagetobufferreceive4vpu
    global message2copyimagetobuffersend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4vpu" 
    method2print = "manoeuvre2copyimagetobuffer4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4vpu-started: " + str(manoeuvre2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4vpu-to-manoeuvre2copyimagetobuffer4vpu: " + str(queue2copyimagetobuffermanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4vpu-to-manoeuvre2copyimagetobuffer4vpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4vpu-to-manoeuvre2copyimagetobuffer4vpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4vpu-try-by-manoeuvre2copyimagetobuffer4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4vpu is None:
            state2copyimagetobuffer4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(state2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4vpu-used-by-manoeuvre2copyimagetobuffer4vpu: " + str(state2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(state2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(ack2copyimagetobufferinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4vpu     = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4vpu     = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4vpu = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4vpu    = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4vpu-initialized-by-manoeuvre2copyimagetobuffer4vpu to: " + str( max2copyimagetobufferstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4vpu = state2copyimagetobuffer4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4vpu-added-by-manoeuvre2copyimagetobuffer4vpu: " + str(state2copyimagetobuffer4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4vpu > max2copyimagetobufferstate4vpu:
        state2copyimagetobuffer4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(state2copyimagetobuffer4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(ack2copyimagetobufferinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4vpu     = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4vpu     = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4vpu = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobufferreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4vpu    = '<default4xi>default2copyimagetobuffer4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str( max2copyimagetobufferstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4vpu = consumerchannel2copyimagetobuffermanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4vpu-received-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobufferreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4vpu == '<copy4xi>copy2copyimagetobuffer4vpu</copy4xi>': 
                message2copyimagetobuffersend4vpu = '<roger4xi>roger2copyimagetobuffer4vpu</roger4xi>'
            else:
                message2copyimagetobuffersend4vpu = '<negative4xi><negative2copyimagetobuffer4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4vpu.send(message2copyimagetobuffersend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4vpu-sent-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobufferreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4vpu = consumerchannel2copyimagetobuffermanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4vpu-received-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobufferreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4vpu == '<ready4xi>ready2copyimagetobuffer4vpu</ready4xi>':
                message2copyimagetobuffersend4vpu = '<ready4xi>ready2copyimagetobuffer4vpu</ready4xi>'
            else:
                message2copyimagetobuffersend4vpu = '<negative4xi><negative2copyimagetobuffer4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4vpu.send(message2copyimagetobuffersend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4vpu-sent-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobufferreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2copyimagetobuffer4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2copyimagetobuffermanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2copyimagetobuffer4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(queue2copyimagetobuffermanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4vpu is True:
                message2copyimagetobuffersend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2copyimagetobuffersend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4vpu-set-by-manoeuvre2copyimagetobuffer4vpu to: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4vpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4vpu-calling-by-manoeuvre2copyimagetobuffer4vpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4vpu-passing-to-send-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4vpu.send(message2copyimagetobuffersend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4vpu-sent-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4vpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4vpu-failed-by-manoeuvre2copyimagetobuffer4vpu: " + str(message2copyimagetobuffersend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4VPU']        = ack2copyimagetobufferinfo4vpu
    response2manouvre4copyimagetobuffer['CODE2ACK4VPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4VPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4VPU']     = message2copyimagetobufferget4vpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4VPU']     = message2copyimagetobufferput4vpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4VPU'] = message2copyimagetobufferreceive4vpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4VPU']    = message2copyimagetobuffersend4vpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4VPU'] = state2copyimagetobuffer4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4vpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4VPU': '<roger4xi>roger2copyimagetobuffer4vpu</roger4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copyimagetobuffer4vpu</copy4xi>',
    #   'CODE2ACK4VPU': ' 0D-1C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4VPU': '<ready4xi>ready2copyimagetobuffer4vpu</ready4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copyimagetobuffer4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-2C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>',
    #   'ACK2INFO4VPU': True,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copyimagetobuffer4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'CODE2ACK4VPU': ' 0D-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copyimagetobuffer4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4vpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4VPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4vpu is True:
#       print("(Step 82A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4vpu(queue4copybuffertoimage4vpu, channel4copybuffertoimageconsume4vpu, channel4copybuffertoimagepublish4vpu)
#           print("(Step 82B/700) OF VPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4VPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4VPU': '<roger4xi>roger2copybuffertoimage4vpu</roger4xi>',
#           #   'ACK2INFO4VPU': False,
#           #   'MESSAGE2GET4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
#           #   'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copybuffertoimage4vpu</copy4xi>',
#           #   'CODE2ACK4VPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
#           #   'CODE2NACK4VPU': ' ',
#           #   'STATE2MANOEUVRE4VPU': 1}
#       except:
#           print("(Step 82Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4vpu(queue2copybuffertoimagemanoeuvre4vpu, consumerchannel2copybuffertoimagemanoeuvre4vpu, publisherchannel2copybuffertoimagemanoeuvre4vpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4vpu
    '''
    global state2copybuffertoimage4vpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4vpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4vpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4vpu                                 
    global message2copybuffertoimageput4vpu
    global message2copybuffertoimagereceive4vpu
    global message2copybuffertoimagesend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4vpu" 
    method2print = "manoeuvre2copybuffertoimage4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4vpu-started: " + str(manoeuvre2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4vpu-to-manoeuvre2copybuffertoimage4vpu: " + str(queue2copybuffertoimagemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4vpu-to-manoeuvre2copybuffertoimage4vpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4vpu-to-manoeuvre2copybuffertoimage4vpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4vpu-try-by-manoeuvre2copybuffertoimage4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4vpu is None:
            state2copybuffertoimage4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(state2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4vpu-used-by-manoeuvre2copybuffertoimage4vpu: " + str(state2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(state2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(ack2copybuffertoimageinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4vpu     = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimageget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4vpu     = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimageput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4vpu = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimagereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4vpu    = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4vpu-initialized-by-manoeuvre2copybuffertoimage4vpu to: " + str( max2copybuffertoimagestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4vpu = state2copybuffertoimage4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4vpu-added-by-manoeuvre2copybuffertoimage4vpu: " + str(state2copybuffertoimage4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4vpu > max2copybuffertoimagestate4vpu:
        state2copybuffertoimage4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(state2copybuffertoimage4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(ack2copybuffertoimageinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4vpu     = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimageget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4vpu     = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimageput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4vpu = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimagereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4vpu    = '<default4xi>default2copybuffertoimage4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str( max2copybuffertoimagestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4vpu = consumerchannel2copybuffertoimagemanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4vpu-received-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4vpu == '<copy4xi>copy2copybuffertoimage4vpu</copy4xi>':
                message2copybuffertoimagesend4vpu = '<roger4xi>roger2copybuffertoimage4vpu</roger4xi>'
            else:
                message2copybuffertoimagesend4vpu = '<negative4xi><negative2copybuffertoimage4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4vpu.send(message2copybuffertoimagesend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4vpu-sent-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4vpu = consumerchannel2copybuffertoimagemanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4vpu-received-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4vpu == '<ready4xi>ready2copybuffertoimage4vpu</ready4xi>':
                message2copybuffertoimagesend4vpu = '<ready4xi>ready2copybuffertoimage4vpu</ready4xi>'
            else:
                message2copybuffertoimagesend4vpu = '<negative4xi><negative2copybuffertoimage4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4vpu.send(message2copybuffertoimagesend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4vpu-sent-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2copybuffertoimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2copybuffertoimagemanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2copybuffertoimage4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(queue2copybuffertoimagemanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4vpu is True:
                message2copybuffertoimagesend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2copybuffertoimagesend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4vpu-set-by-manoeuvre2copybuffertoimage4vpu to: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4vpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4vpu-calling-by-manoeuvre2copybuffertoimage4vpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4vpu-passing-to-send-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4vpu.send(message2copybuffertoimagesend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4vpu-sent-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4vpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4vpu-failed-by-manoeuvre2copybuffertoimage4vpu: " + str(message2copybuffertoimagesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4VPU']        = ack2copybuffertoimageinfo4vpu
    response2manouvre4copybuffertoimage['CODE2ACK4VPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4VPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4VPU']     = message2copybuffertoimageget4vpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4VPU']     = message2copybuffertoimageput4vpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4VPU'] = message2copybuffertoimagereceive4vpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4VPU']    = message2copybuffertoimagesend4vpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4VPU'] = state2copybuffertoimage4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4vpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4VPU': '<roger4xi>roger2copybuffertoimage4vpu</roger4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2copybuffertoimage4vpu</copy4xi>',
    #   'CODE2ACK4VPU': ' 0D-1C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4VPU': '<ready4xi>ready2copybuffertoimage4vpu</ready4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copybuffertoimage4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-2C-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>',
    #   'ACK2INFO4VPU': True,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2copybuffertoimage4vpu</ready4xi>',
    #   'CODE2ACK4VPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'ACK2INFO4VPU': False,
    #   'MESSAGE2GET4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'MESSAGE2RECEIVE4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'CODE2ACK4VPU': ' 0D-',
    #   'MESSAGE2PUT4VPU': '<default4xi>default2copybuffertoimage4vpu</default4xi>',
    #   'CODE2NACK4VPU': ' ',
    #   'STATE2MANOEUVRE4VPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4vpu" usage = "control function for read2image4vpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4vpu(queue2readmanoeuvre4vpu, consumerchannel2readmanoeuvre4vpu, publisherchannel2readmanoeuvre4vpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4vpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4vpu                                                                                        # MAX FOR STATE
    global ack2readinfo4vpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4vpu                                 
    global message2readput4vpu
    global message2readreceive4vpu
    global message2readsend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4vpu" 
    method2print = "manoeuvre2readimage4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4vpu-started: " + str(manoeuvre2readimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4vpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4vpu-to-manoeuvre2readimage4vpu: " + str(queue2readmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4vpu-to-manoeuvre2readimage4vpu: " + str(consumerchannel2readmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4vpu-to-manoeuvre2readimage4vpu: " + str(publisherchannel2readmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4vpu-try-by-manoeuvre2readimage4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4vpu is None:
            state2read4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(state2read4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4vpu-used-by-manoeuvre2readimage4vpu: " + str(state2read4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(state2read4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(ack2readinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4vpu     = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(message2readget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4vpu</default4xi>
        message2readput4vpu     = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(message2readput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        message2readreceive4vpu = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(message2readreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        message2readsend4vpu    = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        max2readstate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4vpu-initialized-by-manoeuvre2readimage4vpu to: " + str( max2readstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4vpu = state2read4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4vpu-added-by-manoeuvre2readimage4vpu: " + str(state2read4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4vpu > max2readstate4vpu:
        state2read4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4vpu-set-by-manoeuvre2readimage4vpu to: " + str(state2read4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4vpu-set-by-manoeuvre2readimage4vpu to: " + str(ack2readinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4vpu     = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4vpu-set-by-manoeuvre2readimage4vpu to: " + str(message2readget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4vpu</default4xi>
        message2readput4vpu     = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4vpu-set-by-manoeuvre2readimage4vpu to: " + str(message2readput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        message2readreceive4vpu = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4vpu-set-by-manoeuvre2readimage4vpu to: " + str(message2readreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        message2readsend4vpu    = '<default4xi>default2read4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4vpu-set-by-manoeuvre2readimage4vpu to: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4vpu</default4xi>
        max2readstate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4vpu-set-by-manoeuvre2readimage4vpu to: " + str( max2readstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4vpu = consumerchannel2readmanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4vpu-received-by-manoeuvre2readimage4vpu: " + str(message2readreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4vpu</copy4xi>
                         
            if message2readreceive4vpu == '<copy4xi>copy2read4vpu</copy4xi>':
                message2readsend4vpu = '<roger4xi>roger2read4vpu</roger4xi>'
            else:
                message2readsend4vpu = '<negative4xi><negative2read4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4vpu.send(message2readsend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4vpu-sent-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4vpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4vpu-failed-by-manoeuvre2readimage4vpu: " + str(message2readreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4vpu = consumerchannel2readmanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4vpu-received-by-manoeuvre2readimage4vpu: " + str(message2readreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4vpu</ready4xi> 
                         
            if message2readreceive4vpu == '<ready4xi>ready2read4vpu</ready4xi>':
                message2readsend4vpu = '<ready4xi>ready2read4vpu</ready4xi>'
            else:
                message2readsend4vpu = '<negative4xi><negative2ready4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4vpu.send(message2readsend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4vpu-sent-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4vpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4vpu-failed-by-manoeuvre2readimage4vpu: " + str(message2readreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2readimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2readmanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2readimage4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4vpu-failed-by-manoeuvre2readimage4vpu: " + str(queue2readmanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4vpu is True:
                message2readsend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2readsend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4vpu-set-by-manoeuvre2readimage4vpu to: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4vpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4vpu-failed-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4vpu-calling-by-manoeuvre2readimage4vpu: " + str(publisherchannel2readmanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4vpu-passing-to-send-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4vpu-failed-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4vpu.send(message2readsend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4vpu-sent-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4vpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4vpu-failed-by-manoeuvre2readimage4vpu: " + str(message2readsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4VPU']        = ack2readinfo4vpu
    response2manouvre4read['CODE2ACK4VPU']        = code2ack
    response2manouvre4read['CODE2NACK4VPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4VPU']     = message2readget4vpu
    response2manouvre4read['MESSAGE2PUT4VPU']     = message2readput4vpu
    response2manouvre4read['MESSAGE2RECEIVE4VPU'] = message2readreceive4vpu
    response2manouvre4read['MESSAGE2SEND4VPU']    = message2readsend4vpu
    response2manouvre4read['STATE2MANOEUVRE4VPU'] = state2read4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4vpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4VPU': '<roger4xi>roger2read4vpu</roger4xi>',
    #  'ACK2INFO4VPU': False,
    #  'MESSAGE2GET4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2read4vpu</copy4xi>',
    #  'CODE2ACK4VPU': ' 0D-1C-',
    #  'MESSAGE2PUT4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4VPU': '<ready4xi>ready2read4vpu</ready4xi>',
    #  'ACK2INFO4VPU': False,
    #  'MESSAGE2GET4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2read4vpu</ready4xi>',
    #  'CODE2ACK4VPU': ' 0D-2C-',
    #  'MESSAGE2PUT4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>',
    #  'ACK2INFO4VPU': True,
    #  'MESSAGE2GET4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2read4vpu</ready4xi>',
    #  'CODE2ACK4VPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'CODE2NACK4VPU': ' ',
    #  'STATE2MANOEUVRE4VPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'ACK2INFO4VPU': False,
    #  'MESSAGE2GET4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'MESSAGE2RECEIVE4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'CODE2ACK4VPU': ' 0D-',
    #  'MESSAGE2PUT4VPU': '<default4xi>default2read4vpu</default4xi>',
    #  'CODE2NACK4VPU': ' ',
    #  'STATE2MANOEUVRE4VPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4vpu" usage = "control function for write2image4vpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4vpu(queue2writemanoeuvre4vpu, consumerchannel2writemanoeuvre4vpu, publisherchannel2writemanoeuvre4vpu):
    '''
    manoeuvres control and command
    '''
    global state2write4vpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4vpu                                                    # MAX FOR STATE
    global ack2writeinfo4vpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4vpu                                 
    global message2writeput4vpu
    global message2writereceive4vpu
    global message2writesend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4vpu" 
    method2print = "manoeuvre2writeimage4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4vpu-started: " + str(manoeuvre2writeimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4vpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4vpu-to-manoeuvre2writeimage4vpu: " + str(queue2writemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4vpu-to-manoeuvre2writeimage4vpu: " + str(consumerchannel2writemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4vpu-to-manoeuvre2writeimage4vpu: " + str(publisherchannel2writemanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4vpu-try-by-manoeuvre2writeimage4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4vpu is None:
            state2write4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(state2write4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4vpu-used-by-manoeuvre2writeimage4vpu: " + str(state2write4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(state2write4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(ack2writeinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4vpu     = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(message2writeget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4vpu</default4xi>
        message2writeput4vpu     = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(message2writeput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        message2writereceive4vpu = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(message2writereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        message2writesend4vpu    = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        max2writestate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4vpu-initialized-by-manoeuvre2writeimage4vpu to: " + str( max2writestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4vpu = state2write4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4vpu-added-by-manoeuvre2writeimage4vpu: " + str(state2write4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4vpu > max2writestate4vpu:
        state2write4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(state2write4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(ack2writeinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4vpu     = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(message2writeget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4vpu</default4xi>
        message2writeput4vpu     = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(message2writeput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        message2writereceive4vpu = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(message2writereceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        message2writesend4vpu    = '<default4xi>default2write4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4vpu</default4xi>
        max2writestate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4vpu-set-by-manoeuvre2writeimage4vpu to: " + str( max2writestate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4vpu = consumerchannel2writemanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4vpu-received-by-manoeuvre2writeimage4vpu: " + str(message2writereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4vpu</copy4xi>
                         
            if message2writereceive4vpu == '<copy4xi>copy2write4vpu</copy4xi>':
                message2writesend4vpu = '<roger4xi>roger2write4vpu</roger4xi>'
            else:
                message2writesend4vpu = '<negative4xi><negative2write4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4vpu.send(message2writesend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4vpu-sent-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4vpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(message2writereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4vpu = consumerchannel2writemanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4vpu-received-by-manoeuvre2writeimage4vpu: " + str(message2writereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4vpu</ready4xi>
                         
            if message2writereceive4vpu == '<ready4xi>ready2write4vpu</ready4xi>':
                message2writesend4vpu = '<ready4xi>ready2write4vpu</ready4xi>'
            else:
                message2writesend4vpu = '<negative4xi><negative2ready4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4vpu.send(message2writesend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4vpu-sent-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4vpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(message2writereceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2writeimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2writemanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2writeimage4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(queue2writemanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4vpu is True:
                message2writesend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2writesend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4vpu-set-by-manoeuvre2writeimage4vpu to: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4vpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4vpu-calling-by-manoeuvre2writeimage4vpu: " + str(publisherchannel2writemanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4vpu-passing-to-send-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4vpu.send(message2writesend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4vpu-sent-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4vpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4vpu-failed-by-manoeuvre2writeimage4vpu: " + str(message2writesend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4VPU']        = ack2writeinfo4vpu
    response2manouvre4write['CODE2ACK4VPU']        = code2ack
    response2manouvre4write['CODE2NACK4VPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4VPU']     = message2writeget4vpu
    response2manouvre4write['MESSAGE2PUT4VPU']     = message2writeput4vpu
    response2manouvre4write['MESSAGE2RECEIVE4VPU'] = message2writereceive4vpu
    response2manouvre4write['MESSAGE2SEND4VPU']    = message2writesend4vpu
    response2manouvre4write['STATE2MANOEUVRE4VPU'] = state2write4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4vpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4VPU': '<roger4xi>roger2write4vpu</roger4xi>', 
    #  'ACK2INFO4VPU': False, 'MESSAGE2GET4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2write4vpu</copy4xi>', 
    #  'CODE2ACK4VPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4VPU': '<ready4xi>ready2write4vpu</ready4xi>', 
    #  'ACK2INFO4VPU': False, 'MESSAGE2GET4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2write4vpu</ready4xi>', 
    #  'CODE2ACK4VPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>', 
    #  'ACK2INFO4VPU': True, 'MESSAGE2GET4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2write4vpu</ready4xi>', 
    #  'CODE2ACK4VPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2write4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4vpu" usage = "control function for map2image4vpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4vpu(queue2mapmanoeuvre4vpu, consumerchannel2mapmanoeuvre4vpu, publisherchannel2mapmanoeuvre4vpu):
    '''
    manoeuvres control and command
    '''
    global state2map4vpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4vpu                                                    # MAX FOR STATE
    global ack2mapinfo4vpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4vpu                                 
    global message2mapput4vpu
    global message2mapreceive4vpu
    global message2mapsend4vpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4vpu" 
    method2print = "manoeuvre2mapimage4vpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4vpu-started: " + str(manoeuvre2mapimage4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4vpu-to-manoeuvre2mapimage4vpu: " + str(queue2mapmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4vpu-to-manoeuvre2mapimage4vpu: " + str(consumerchannel2mapmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4vpu-to-manoeuvre2mapimage4vpu: " + str(publisherchannel2mapmanoeuvre4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4vpu-try-by-manoeuvre2mapimage4vpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4vpu is None:
            state2map4vpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(state2map4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4vpu-used-by-manoeuvre2mapimage4vpu: " + str(state2map4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4vpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(state2map4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4vpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(ack2mapinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4vpu     = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(message2mapget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4vpu</default4xi>
        message2mapput4vpu     = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(message2mapput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        message2mapreceive4vpu = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(message2mapreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        message2mapsend4vpu    = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        max2mapstate4vpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4vpu-initialized-by-manoeuvre2mapimage4vpu to: " + str( max2mapstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4vpu = state2map4vpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4vpu-added-by-manoeuvre2mapimage4vpu: " + str(state2map4vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4vpu > max2mapstate4vpu:
        state2map4vpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(state2map4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4vpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(ack2mapinfo4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4vpu     = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(message2mapget4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4vpu</default4xi>
        message2mapput4vpu     = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(message2mapput4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        message2mapreceive4vpu = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(message2mapreceive4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        message2mapsend4vpu    = '<default4xi>default2map4vpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4vpu</default4xi>
        max2mapstate4vpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4vpu-set-by-manoeuvre2mapimage4vpu to: " + str( max2mapstate4vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4vpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4vpu = consumerchannel2mapmanoeuvre4vpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4vpu-received-by-manoeuvre2mapimage4vpu: " + str(message2mapreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4vpu == '<copy4xi>copy2mapimage4vpu</copy4xi>':
                message2mapsend4vpu = '<roger4xi>roger2mapimage4vpu</roger4xi>'
            else:
                message2mapsend4vpu = '<negative4xi><negative2mapimage4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4vpu.send(message2mapsend4vpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4vpu-sent-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(message2mapreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4vpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4vpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4vpu = consumerchannel2mapmanoeuvre4vpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4vpu-received-by-manoeuvre2mapimage4vpu: " + str(message2mapreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4vpu == '<ready4xi>ready2mapimage4vpu</ready4xi>':
                message2mapsend4vpu = '<ready4xi>ready2mapimage4vpu</ready4xi>'
            else:
                message2mapsend4vpu = '<negative4xi><negative2ready4vpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4vpu.send(message2mapsend4vpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4vpu-sent-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(message2mapreceive4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4vpu == 3:    
        info2image4vpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4vpu-try-by-manoeuvre2mapimage4vpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4vpu = queue2mapmanoeuvre4vpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4vpu-got-by-manoeuvre2mapimage4vpu: " + str(info2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4vpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4vpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4vpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(queue2mapmanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4vpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4vpu is True:
                message2mapsend4vpu = '<ack4xi>ack2imageinfo4vpu</ack4xi>'
            else:
                message2mapsend4vpu = '<nack4xi>nack2imageinfo4vpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4vpu-set-by-manoeuvre2mapimage4vpu to: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4vpu-calling-by-manoeuvre2mapimage4vpu: " + str(publisherchannel2mapmanoeuvre4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4vpu-passing-to-send-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4vpu.send(message2mapsend4vpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4vpu-sent-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4vpu-failed-by-manoeuvre2mapimage4vpu: " + str(message2mapsend4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4VPU']        = ack2mapinfo4vpu
    response2manouvre4map['CODE2ACK4VPU']        = code2ack
    response2manouvre4map['CODE2NACK4VPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4VPU']     = message2mapget4vpu
    response2manouvre4map['MESSAGE2PUT4VPU']     = message2mapput4vpu
    response2manouvre4map['MESSAGE2RECEIVE4VPU'] = message2mapreceive4vpu
    response2manouvre4map['MESSAGE2SEND4VPU']    = message2mapsend4vpu
    response2manouvre4map['STATE2MANOEUVRE4VPU'] = state2map4vpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4vpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4VPU': '<roger4xi>roger2mapimage4vpu</roger4xi>', 
    #  'ACK2INFO4VPU': False, 
    #  'MESSAGE2GET4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<copy4xi>copy2mapimage4vpu</copy4xi>', 
    #  'CODE2ACK4VPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 1}
     
    #2 {'MESSAGE2SEND4VPU': '<ready4xi>ready2mapimage4vpu</ready4xi>', 
    #  'ACK2INFO4VPU': False, 
    #  'MESSAGE2GET4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2mapimage4vpu</ready4xi>', 
    #  'CODE2ACK4VPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4VPU': '<ack4xi>ack2imageinfo4vpu</ack4xi>', 
    #  'ACK2INFO4VPU': True, 
    #  'MESSAGE2GET4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<ready4xi>ready2mapimage4vpu</ready4xi>', 
    #  'CODE2ACK4VPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 
    #  'STATE2MANOEUVRE4VPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'ACK2INFO4VPU': False, 
    #  'MESSAGE2GET4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'MESSAGE2RECEIVE4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'CODE2ACK4VPU': ' 0D-', 
    #  'MESSAGE2PUT4VPU': '<default4xi>default2map4vpu</default4xi>', 
    #  'CODE2NACK4VPU': ' ', 'STATE2MANOEUVRE4VPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4vpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4vpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4VPU : ', response4write2image)
#       queue2write4vpu      = response4write2image['QUEUE']
#       channel2publish4vpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4vpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4VPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2writesend4vpu = "<copy4xi>copy2write4vpu</copy4xi>"
#       channel2publish4vpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4vpu.send(message2writesend4vpu)
#       print("(Step 17B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2publish4vpu, " MESSAGE2SEND4VPU: " , message2writesend4vpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4VPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2write4vpu, channel2consume4vpu, channel2publish4vpu)
#       print("(Step 18D/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 18Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:
#       channel2consume4vpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4vpu = channel2consume4vpu.recv()
#       print("(Step 19B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2consume4vpu, " CONTROL & COMMAND: ", message2writereceive4vpu)                                                                                                  
#       # <roger4xi>roger2write4vpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4VPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4VPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF VPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:                               
#       message2writesend4vpu = "<write4xi>write2write4vpu</write4xi>"
#       channel2publish4vpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4vpu.send(message2writesend4vpu)
#       print("(Step 20B/700) OF VPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4VPU: ", channel2publish4vpu, " MESSAGE2SEND4VPU: " , message2writesend4vpu)                                                                                                
#       # <write4xi>write2write4vpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4VPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2write4vpu, channel2consume4vpu, channel2publish4vpu)
#       print("(Step 21B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 21Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4VPU ', time4xi())     
#   try:
#       channel2consume4vpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4vpu = channel2consume4vpu.recv()
#       print("(Step 22B/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2consume4vpu, " CONTROL & COMMAND: ", message2writereceive4vpu)                                                                                                  
#       # <write4xi>write2write4vpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4VPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4VPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2write4vpu, channel2consume4vpu, channel2publish4vpu)
#       print("(Step 25B/700) OF VPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", manoeuvre2readimage4vpu)
#   except:
#       print("(Step 25Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4vpu = channel2consume4vpu.recv()
#       print("(Step 26D/700) OF VPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4VPU: ", channel2consume4vpu, " CONTROL & COMMAND: ", message2writereceive4vpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4vpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4VPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF VPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi()) 
#   try:        
#       response2manoeuvre4vpu = manoeuvre2readimage4vpu(queue2write4vpu, channel2consume4vpu, channel2publish4vpu)
#       print("(Step 27B/700) OF VPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4VPU ", response2manoeuvre4vpu)
#   except:
#       print("(Step 27Z/700) OF VPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4VPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4vpu(task4imagewrite2vpu=None):

    '''
    writes 2D images for VPU
    '''
    global vpu2image4xi                                                     # 2D-IMAGES FOR VPU  
    global control2writeimage4vpu                                              # COMMAND AND CONTROL FOR VPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4vpu"
    method2print = "write2image4vpu"

    message2print = "{(Step 1A/33) started" + "; write2image4vpu-started: " + str(write2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4vpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2vpu-to-write2image4vpu: " + str(task4imagewrite2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE VPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                              # INITIALIZE IMAGE FOR VPU 
            message2print = "{(Step 1C/70) created" + "; vpu2image4xi-created-by-write2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4vpu-skipped-by-write2image4vpu: " + str(write2image4vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4vpu-skipped-by-write2image4vpu: <function write2image4vpu at 0x00000000038E8EC8>                                 
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 1E/70) set" + "; vpu2image4xi-set-by-write2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            vpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; vpu2image4xi-yielded-by-write2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2vpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2vpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2vpu-created-by-write2image4vpu to: " + str(task4imagewrite2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                                 # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4vpu-created-by-write2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4vpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4vpu"                                                            # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4vpu"                                                            # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4vpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4vpu', 
            #  'METHOD': 'write2image4vpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2vpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2vpu-created-by-write2image4vpu to: " + str(task4imagewrite2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4vpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4vpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4vpu',
            #  'METHOD': 'write2image4vpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2vpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2vpu-trying-by-write2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2vpu-got-by-write2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4vpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2vpu-failed-by-write2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2vpu ['CONTEXT'] = context2vpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2vpu-trying-by-write2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2vpu-got-by-write2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2vpu-failed-by-write2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2vpu ['DEVICE'] = device2vpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2vpu-trying-by-write2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2vpu-got-by-write2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4vpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4vpu', 
                #  'METHOD': 'create2context4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4vpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4vpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2vpu-failed-by-write2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2vpu ['IMAGE'] = imageinfo2vpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2vpu-trying-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2vpu-got-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2vpu-failed-by-write2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2vpu ['QUEUE'] = queue2vpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2vpu-trying-by-write2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2vpu-got-by-write2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2vpu-failed-by-write2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2vpu ['PIPE'] = pipe2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2vpu-trying-by-write2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2vpu-got-by-write2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4vpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4vpu', 
                #  'METHOD': 'create2commandqueue4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4vpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2vpu-failed-by-write2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2vpu-trying-by-write2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                               # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2vpu-got-by-write2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4vpu', 
                #  'METHOD': 'create2buffer4vpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2vpu-failed-by-write2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2vpu ['BUFFER'] = bufferinfo2vpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2vpu-trying-by-write2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2vpu-got-by-write2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4vpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4vpu', 
                #  'METHOD': 'create2event4vpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4vpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2vpu-failed-by-write2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2vpu ['EVENT'] = eventinfo2vpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2vpu-trying-by-write2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2vpu-set-by-write2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2vpu-failed-by-write2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-write2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2vpu-set-by-write2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-write2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2vpu-trying-by-write2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2vpu-set-by-write2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2vpu-failed-by-write2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2vpu ['BLOCKING'] = blocking2vpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2vpu-trying-by-write2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2vpu-set-by-write2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2vpu-failed-by-write2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2vpu ['ORIGIN'] = origin2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2vpu-trying-by-write2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2vpu-set-by-write2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2vpu-failed-by-write2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2vpu ['REGION'] = region2vpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2vpu = 800
            width2vpu = 800
            rowpitch2vpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2vpu-trying-by-write2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2vpu-set-by-write2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2vpu-failed-by-write2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2vpu ['ROWPITCH'] = rowpitch2vpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2vpu = None
            size2vpu = int(width2vpu * height2vpu)                                                                  # CALCULATED MEMORY
            if size2vpu == 0:
                size2vpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2vpu-trying-by-write2image4vpu: " + str(pointer2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2vpu = Arena4Xi(size2vpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2vpu-got-by-write2image4vpu: " + str(arena2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2vpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4vpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2vpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4vpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2vpu:
                    size2vpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2vpu-set-by-write2buffer4vpu to: " + str(size2vpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2vpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2vpu-set-by-write2buffer4vpu to: " + str(pointer2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2vpu-failed-by-write2image4vpu: " + str(pointer2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2vpu ['POINTER'] = pointer2vpu                            
            task4imagewrite2vpu ['SIZE']    = size2vpu                                                               # AUTOMATIC BY VPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2vpu-trying-by-write2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2vpu-set-by-write2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2vpu-failed-by-write2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2vpu ['SLICEPITCH'] = slicepitch2vpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2vpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4vpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4vpu = trace2image4vpu                                                             # CALLBACK FUNCTION FOR VPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4vpu-created-by-write2image4vpu to: " + str(function2callback4vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4vpu"                                                        # MODULE OF VPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4vpu"                                                        # FUNCTION OF VPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4vpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR VPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4vpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2vpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO VPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2vpu['FUNCTION2CALLBACK'] = function2callback4vpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2vpu-created-by-write2image4vpu to: " + str(task4imagewrite2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4vpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4vpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4vpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2vpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2vpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2vpu-trying-by-write2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2vpu = get2context4vpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2vpu-got-by-write2image4vpu: " + str(context2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2vpu-failed-by-write2image4vpu: " + str(context2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2vpu ['CONTEXT'] = context2vpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2vpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2vpu-trying-by-write2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2vpu = get2vpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2vpu-got-by-write2image4vpu: " + str(device2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2vpu-failed-by-write2image4vpu: " + str(device2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2vpu ['DEVICE'] = device2vpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2vpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2vpu-trying-by-write2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2vpu = get2imageinfo4vpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2vpu-got-by-write2image4vpu: " + str(imageinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2vpu-failed-by-write2image4vpu: " + str(imageinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2vpu ['IMAGE'] = imageinfo2vpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY VPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2vpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2vpu-trying-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2vpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2vpu-got-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2vpu-failed-by-write2image4vpu: " + str(queue2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2vpu ['QUEUE'] = queue2vpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY VPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2vpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2vpu-trying-by-write2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2vpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2vpu-got-by-write2image4vpu: " + str(pipe2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2vpu-failed-by-write2image4vpu: " + str(pipe2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2vpu ['PIPE'] = pipe2vpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2vpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2vpu-trying-by-write2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2vpu = get2commandqueueinfo4vpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2vpu-got-by-write2image4vpu: " + str(commandqueueinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2vpu-failed-by-write2image4vpu: " + str(commandqueueinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2vpu ['COMMANDQUEUE'] = commandqueueinfo2vpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY VPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2vpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2vpu-trying-by-write2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2vpu = get2bufferinfo4vpu()                                                           # SYSTEM CREATES BUFFER FOR VPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2vpu-got-by-write2image4vpu: " + str(bufferinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2vpu-failed-by-write2image4vpu: " + str(bufferinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2vpu ['BUFFER'] = bufferinfo2vpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2vpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2vpu-trying-by-write2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2vpu = get2eventinfo4vpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2vpu-got-by-write2image4vpu: " + str(eventinfo2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2vpu-failed-by-write2image4vpu: " + str(eventinfo2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2vpu ['EVENT'] = eventinfo2vpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2vpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2vpu-trying-by-write2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2vpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2vpu-set-by-write2image4vpu: " + str(eventwaitlist2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2vpu-failed-by-write2image4vpu: " + str(eventwaitlist2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2vpu ['EVENTWAITLIST'] = eventwaitlist2vpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2vpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2vpu-trying-by-write2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2vpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2vpu = eventwaitlistquantity2vpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2vpu-set-by-write2image4vpu: " + str(eventwaitlistquantity2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2vpu-failed-by-write2image4vpu: " + str(eventwaitlistquantity2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2vpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2vpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2vpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2vpu-trying-by-write2image4vpu: " + str(blocking2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2vpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2vpu-set-by-write2image4vpu to: " + str(blocking2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2vpu-failed-by-write2image4vpu: " + str(blocking2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2vpu ['BLOCKING'] = blocking2vpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2vpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2vpu-trying-by-write2image4vpu: " + str(origin2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2vpu['X'] = 0 
                origin2vpu['Y'] = 0  
                origin2vpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2vpu-set-by-write2image4vpu to: " + str(origin2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2vpu-failed-by-write2image4vpu: " + str(origin2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2vpu ['ORIGIN'] = origin2vpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2vpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2vpu-trying-by-write2image4vpu: " + str(region2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2vpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2vpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2vpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2vpu-set-by-write2image4vpu to: " + str(region2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2vpu-failed-by-write2image4vpu: " + str(region2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2vpu ['REGION'] = region2vpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2vpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2vpu-trying-by-write2image4vpu: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2vpu = int(width2vpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2vpu-set-by-write2image4vpu to: " + str(rowpitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2vpu-failed-by-write2image4vpu: " + str(rowpitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2vpu ['ROWPITCH'] = rowpitch2vpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2vpu = None
            size2vpu = int(width2vpu * height2vpu)                                                                  # CALCULATED MEMORY
            if size2vpu == 0:
                size2vpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2vpu-trying-by-write2image4vpu: " + str(pointer2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2vpu = Arena4Xi(size2vpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2vpu-got-by-write2image4vpu: " + str(arena2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2vpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4vpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2vpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4vpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2vpu:
                    size2vpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2vpu-set-by-write2buffer4vpu to: " + str(size2vpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2vpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2vpu-set-by-write2buffer4vpu to: " + str(pointer2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2vpu-failed-by-write2image4vpu: " + str(pointer2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2vpu ['POINTER'] = pointer2vpu                            
            task4imagewrite2vpu ['SIZE']    = size2vpu                                                               # AUTOMATIC BY VPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2vpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2vpu-trying-by-write2image4vpu: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2vpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2vpu-set-by-write2image4vpu to: " + str(slicepitch2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2vpu-failed-by-write2image4vpu: " + str(slicepitch2vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2vpu ['SLICEPITCH'] = slicepitch2vpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2vpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4vpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2vpu-yielded-by-write2image4vpu: " + str(task4imagewrite2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4vpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2vpu = False
    channels2vpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2vpu-trying-by-write2image4vpu: " + str(channels2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2vpu, publisherchannel2vpu = task4imagewrite2vpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2vpu-got-by-write2image4vpu: " + str(consumerchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2vpu-got-by-write2image4vpu: " + str(publisherchannel2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2vpu = True
        
        control2writeimage4vpu['CONSUMERCHANNEL']  = consumerchannel2vpu
        
        
        control2writeimage4vpu['PUBLISHERCHANNEL'] = publisherchannel2vpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2vpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2vpu = False
    queue2vpu = None
    message2print = "{(Step 41A/70) trying" + "queue2vpu-trying-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2vpu = task4imagewrite2vpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2vpu-got-by-write2image4vpu: " + str(queue2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2vpu = True
        control2writeimage4vpu['QUEUE']  = queue2vpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2vpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4vpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4vpu-trying-by-write2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4vpu = DummyProcess4Xi(target=manoeuvre2readimage4vpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2vpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2vpu, publisherchannel2vpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4vpu-created-by-write2image4vpu: " + str(dummy2image4vpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4vpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4vpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4vpu-returning-by-write2image4vpu: " + str(control2writeimage4vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4vpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4vpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF VPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF VPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4vpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF VPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF VPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF VPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4vpu(task4release2vpu=None):

    '''
    releases vpu2image4xi
    '''
    global vpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4vpu"
    method2print = "release2image4vpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4vpu-started: " + str(release2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4vpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2vpu-to-release2image4vpu: " + str(task4release2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            vpu2image4xi = create2image4vpu()
        except:
            vpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; vpu2image4xi-yielded-by-release2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4vpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2vpu is None:
        task4release2vpu = {}
        task4release2vpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        vpu2image4xi['COUNT2REFERENCE'] = vpu2image4xi['COUNT2REFERENCE'] - task4release2vpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4vpu: " + str(vpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if vpu2image4xi['COUNT2REFERENCE'] < 0:
            vpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4vpu: " + " ** " + str(vpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4vpu : release2image4vpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4vpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            vpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4vpu: " + " ** " + str(vpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; vpu2image4xi-yielded-by-release2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    vpu2image4xi ['ACK2RETURN']  = code2ack
    vpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; vpu2image4xi-returning-by-release2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4vpu',
    #  'METHOD': 'create2image4vpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4vpu', 'METHOD': 'create2buffer4vpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return vpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4vpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF VPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4vpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF VPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF VPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4vpu(task4retain2vpu=None):

    '''
    retains either vpu2image4xi
    '''
    global vpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4vpu"
    method2print = "retain2image4vpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4vpu-started: " + str(retain2image4vpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4vpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2vpu-to-retain2image4vpu: " + str(task4retain2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2vpu is None:
        task4retain2vpu = {}
        task4retain2vpu['DIMENSION'] = 2
        task4retain2vpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2vpu-set-by-retain2image4vpu to: " + str(task4retain2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2vpu.keys():
        if task4retain2vpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2vpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4vpu to: " + str(task4retain2vpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2vpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2vpu.keys():
        pass
    else:
        task4retain2vpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4vpu to: " + str(task4retain2vpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2vpu['COUNT2REFERENCE'] < 1:
        task4retain2vpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4vpu to: " + str(task4retain2vpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if vpu2image4xi is None:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 4A/70) created" + "; vpu2image4xi-created-by-retain2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            vpu2image4xi = create2image4vpu()
            message2print = "{(Step 4B/70) created" + "; vpu2image4xi-created-by-retain2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            vpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; vpu2image4xi-yielded-by-retain2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        vpu2image4xi['COUNT2REFERENCE'] = vpu2image4xi['COUNT2REFERENCE'] + task4retain2vpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4vpu: " + str(vpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if vpu2image4xi['COUNT2REFERENCE'] <= 0:
            vpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4vpu: " + " ** " + str(vpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            vpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4vpu: " + " ** " + str(vpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; vpu2image4xi-yielded-by-retain2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    vpu2image4xi ['ACK2RETURN']  = code2ack
    vpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; vpu2image4xi-returning-by-retain2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4vpu', 'METHOD': 'create2image4vpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return vpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4vpu" usage = "
#   #1
#   print('(Step 3A/33) OF VPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR VPU', time4xi())  
#   try:
#       imageinfo2vpu = get2imageinfo4vpu()                                                                       
#       print('(Step 3B/33) OF VPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR VPU: ', imageinfo2vpu) 
#   except:
#       print('(Step 3Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR VPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF VPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF VPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4vpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF VPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF VPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF VPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4vpu(task4getinfo2image=None):

    '''
    gets vpu2image4xi
    '''
    
    global vpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4vpu"
    method2print = "get2imageinfo4vpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4vpu-started: " + str(get2imageinfo4vpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4vpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4vpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2vpu = {}
    try: 
        if vpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR VPU           
            # -------------------------------------------------------------------------------------           
            image2vpu = create2image4vpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2vpu-initialized-by-get2imageinfo4vpu to: " + str(image2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; vpu2image4xi-using-by-get2imageinfo4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4vpu', 
            #  'METHOD': 'create2image4vpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF VPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2vpu = vpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2vpu-got-by-get2imageinfo4vpu: " + str(image2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = vpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = vpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4vpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4vpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2vpu [key2task] = vpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2vpu-set-by-get2imageinfo4vpu to: " + str(image2vpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4vpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4vpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; vpu2image4xi-using-by-get2image4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2vpu = create2image4vpu()
            message2print = "{(Step 1F/33) initialized" + "; image2vpu-initialized-by-get2imageinfo4vpu to: " + str(image2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2vpu = vpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2vpu-got-by-get2imageinfo4vpu: " + str(image2vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4vpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2vpu [key2task] = vpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2vpu-set-by-get2imageinfo4vpu to: " + str(image2vpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4vpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2vpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2vpu-yielded-by-get2image4vpu: " + str(image2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2vpu-returning-by-get2imageinfo4vpu: " + str(image2vpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4vpu', 
    #  'METHOD': 'create2image4vpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4vpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4vpu', 
    #  'METHOD': 'create3image4vpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2vpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4vpu" usage = "
#
#   print('(Step 3A/33) OF VPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR VPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4vpu()                                                                       
#       print('(Step 3B/33) OF VPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR VPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR VPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4vpu(task4supportedimageformats2vpu=None):  

    '''
    gets supported image formats for vpu
    '''
    
    global vpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4vpu"
    method2print = "get2supportedimageformats4vpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4vpu-started: " + str(get2supportedimageformats4vpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4vpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2vpu-passed-to-get2supportedimageformats4vpu: " + str(task4supportedimageformats2vpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2vpu is None:
        task4supportedimageformats2vpu = {} 
        task4supportedimageformats2vpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2vpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2vpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2vpu = {} 
    try: 
        if vpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2vpu = create2image4vpu()                                                                          # create vpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2vpu-initialized-by-get2supportedimageformats4vpu to: " + str(image2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR VPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2vpu = vpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2vpu-got-by-get2supportedimageformats4vpu: " + str(supportedimageformats2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2vpu = create2image4vpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2vpu-initialized-by-get2supportedimageformats4vpu to: " + str(image2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR VPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2vpu = vpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2vpu-got-by-get2supportedimageformats4vpu: " + str(supportedimageformats2vpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2vpu-yielded-by-get2image4vpu: " + str(supportedimageformats2vpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2vpu-returning-by-get2supportedimageformats4vpu: " + str(supportedimageformats2vpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2vpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4vpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF VPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR VPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2vpu = wait2image4vpu()
#       keys2response = response4wait2vpu.keys()
#       print('(Step 16B/700) OF VPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR VPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2vpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4vpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4vpu', 'METHOD': 'create2image4vpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4vpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR VPU: ', wait2vpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4vpu(task4wait2image=None):

    '''
    waits vpu2image4vpu
    '''
    
    global vpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4vpu"
    method2print = "wait2image4vpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4vpu-started: " + str(wait2image4vpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4vpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if vpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            vpu2image4xi = create2image4vpu()
            
            message2print = "{(Step 1B/33) created" + "; vpu2image4xi-created-by-wait2image4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4vpu = None
            try:
                queue2image4vpu = vpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4vpu-got-by-wait2image4vpu to: " + str(queue2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; vpu2xi-yielded-by-wait2image4vpu: " + str(vpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4vpu = 0
            try:                    
                size2image4vpu = queue2image4vpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4vpu-got-by-wait2image4vpu to: " + str(size2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; vpu2xi-yielded-by-wait2image4vpu: " + str(vpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4vpu = True       
            try:                    
                empty2image4vpu= queue2image4vpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4vpu-got-by-wait2image4vpu to: " + str(empty2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4vpu-yielded-by-wait2image4vpu: " + str(empty2image4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4vpu = False       
            try:                                
                full2image4vpu = queue2image4vpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4vpu-got-by-wait2image4vpu to: " + str(full2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4vpu-yielded-by-wait2image4vpu: " + str(full2image4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; vpu2image4xi-finished-by-wait2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR VPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; vpu2image4xi-used-by-wait2image4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4vpu = None
            try:
                queue2image4vpu = vpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4vpu-got-by-wait2image4vpu to: " + str(queue2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; vpu2xi-yielded-by-wait2image4vpu: " + str(vpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4vpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4vpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4vpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4vpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4vpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4vpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4vpu-warned-by-wait2image4vpu: " + " ** " + str(queue2image4vpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4vpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4vpu = 0        
            try:                    
                size2queue4vpu = queue2image4vpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4vpu-got-by-wait2image4vpu to: " + str(size2queue4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                vpu2image4xi ['SIZE2QUEUE'] = size2queue4vpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4vpu to: " + str(vpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4vpu-yielded-by-wait2image4vpu: " + str(size2queue4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4vpu = False
            try:                    
                empty2queue4vpu = queue2image4vpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4vpu-got-by-wait2image4vpu to: " + str(empty2queue4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                vpu2image4xi ['EMPTY2QUEUE'] = empty2queue4vpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4vpu to: " + str(vpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4vpu-yielded-by-wait2image4vpu: " + str(empty2queue4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4vpu = False
            try:                                
                full2queue4vpu = queue2image4vpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4vpu-got-by-wait2image4vpu to: " + str(full2queue4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                vpu2image4xi ['FULL2QUEUE'] = full2queue4vpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4vpu to: " + str(vpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4vpu-yielded-by-wait2image4vpu: " + str(full2queue4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4vpu = None
            try:                                
                state2queue4vpu = queue2image4vpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4vpu-got-by-wait2image4vpu to: " + str(state2queue4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                vpu2image4xi ['STATE2QUEUE'] = state2queue4vpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4vpu to: " + str(vpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4vpu-yielded-by-wait2image4vpu: " + str(state2queue4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; vpu2image4xi-finished-by-wait2image4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            vpu2image4vpu = create2image4vpu()
            message2print = "{(Step 1R/33) got" + "; vpu2image4xi-got-by-wait2image4vpu to: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4vpu = None
            try:
                queue2image4vpu = vpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4vpu-got-by-wait2image4vpu to: " + str(queue2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4vpu-yielded-by-wait2image4vpu: " + str(queue2image4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4vpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4vpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4vpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4vpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4vpu-warned-by-wait2image4vpu: " + " ** " + str(queue2image4vpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4vpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4vpu = 0                             
            try:                    
                size2image4vpu = queue2image4vpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4vpu-got-by-wait2image4vpu to: " + str(size2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4vpu-yielded-by-wait2image4vpu: " + str(size2image4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4vpu = True          
            try:                    
                empty2image4vpu = queue2image4vpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4vpu-got-by-wait2image4vpu to: " + str(empty2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; vpu2xi-yielded-by-wait2image4vpu: " + str(vpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4vpu = False          
            try:                                
                full2image4vpu= queue2image4vpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4vpu-got-by-wait2image4vpu to: " + str(full2image4vpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4vpu-yielded-by-wait2image4vpu: " + str(full2image4vpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; vpu2image4xi-finished-by-wait2image4vpu: " + str(vpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            vpu2image4vpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; vpu2image4xi-yielded-by-wait2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; vpu2image4xi-returning-by-wait2image4vpu: " + str(vpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return vpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "VPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for VPU 2D images" usage = "2D Image4Xi of VPU"
#>

# =================================================================================================
class VPU2Image4Xi(object): 
    ''' 
    VPU2Image4Xi offers methods for VPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2vpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-VPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF VPU2IMAGE4XI5TEST - CREATING IMAGE4VPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2vpu = VPU2Image4Xi()
    #       print('(Step 5B/33) OF VPU2IMAGE4XI5TEST - CREATED IMAGE4VPU OBJECT: ', imageobject2vpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4VPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2vpu = None):
        '''
        _init__ method initializes VPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "VPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-VPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; vpu2image2xi-of-VPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-vpu2image2xi-of-VPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2vpu-sharing-by-vpu2image2xi-of-VPU2Image4Xi: " + str(VPU2Image4Xi.classvariable2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2vpu-passed-to-init-of-VPU2Image4Xi: " + str(task4imagecreate2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2vpu-passing-to-create2image4vpu-by-init-of-VPU2Image4Xi: " + str(task4imagecreate2vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        vpu2image2xi = create2image4vpu(task4imagecreate2vpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4vpu-performed-by-init-of-VPU2Image4Xi: " + str(create2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4vpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; vpu2image2xi-of-VPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { VPU2Image4Xi : __init__ ; (Step 3A/33) activated; vpu2image2xi-of-VPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-vpu2image2xi-of-VPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2vpu-shared-by-vpu2image2xi-of-VPU2Image4Xi: " + str(VPU2Image4Xi.classvariable2vpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-VPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF VPU2IMAGE4XI5TEST - TRYING TO CREATE VPU IMAGE', time4xi()) 
    #   try:
    #       xi2vpu = VPU2Image4Xi()                            
    #       image2vpu = xi2vpu.create2image()                                  
    #       print('(Step 23B/33) OF VPU2IMAGE4XI5TEST - CREATED VPU IMAGE: ', image2vpu)
    #   except:
    #       print('(Step 23Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO CREATE VPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of VPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-VPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4vpu-by-create2image-of-VPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        vpu2image2xi = create2image4vpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4vpu-performed-by-create2image-of-VPU2Image4Xi: " + str(create2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4vpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; vpu2image2xi-returning-by-create2image-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4vpu',
        #  'METHOD': 'create2image4vpu'},
        #  'WIDTH': 800 ...

        return vpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF VPU2IMAGE4XI - TRYING TO SET VPU IMAGE')
    #   try:
    #       cc2xi = VPU2Image4Xi()                            
    #       image2vpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF VPU4XI5TEST - SET VPU IMAGE: ', image2vpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF VPU4XI5TEST *** FAILED TO SET VPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image VPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-VPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4vpu-by-set2image-of-VPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        vpu2image2xi = set2image4vpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4vpu-performed-by-set2image-of-VPU2Image4Xi: " + str(set2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4vpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; vpu2image2xi-returning-by-set2image-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4vpu', 'METHOD': 'create2image4vpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4vpu',
        #  'METHOD': 'create2buffer4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return vpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-VPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF VPU2IMAGE4XI - TRYING TO GET VPU 2D IMAGE')
    #   try:
    #       cc2xi = VPU2Image4Xi()                            
    #       image2vpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF VPU4XI5TEST - GOT VPU IMAGE: ', image2vpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF VPU4XI5TEST *** FAILED TO GET VPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of VPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        vpu2image2xi = get2imageinfo4vpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; vpu2image2xi-returning-by-get2imageinfo-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4vpu', 'METHOD': 'create2image4vpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4vpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4vpu',
        #  'METHOD': 'create2buffer4vpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4vpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return vpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-VPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF VPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR VPU')
    #  try:
    #      cc2xi = VPU2Image4Xi()                            
    #      supportedimageformats2vpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF VPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR VPU: ', supportedimageformats2vpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF VPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR VPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of VPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2vpu = get2supportedimageformats4vpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2vpu-returning-by-get2imageinfo-of-VPU2Image4Xi: " + str(supportedimageformats2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2vpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2vpu = {}
    #   try:
    #       xi2vpu = VPU2Image4Xi()    
    #       response4read2vpu = xi2vpu.read2image()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2VPU: ', response4read2vpu) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-VPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4vpu-by-read2image-of-VPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2vpu = read2image4vpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4vpu-performed-by-read2image-of-VPU2Image4Xi: " + str(read2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4vpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2vpu-returning-by-read2image-of-VPU2Image4Xi: " + str(response4readimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2vpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2vpu = VPU2Image4Xi() 
    #   ...
    #   response4map2vpu = {}
    #   try:
    #       response4map2vpu = xi2vpu.map2image()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2VPU: ', response4map2vpu) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-VPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4vpu-by-map2image-of-VPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2vpu = map2image4vpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4vpu-performed-by-map2image-of-VPU2Image4Xi: " + str(map2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4vpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2vpu-returning-by-map2image-of-VPU2Image4Xi: " + str(response4mapimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2vpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-VPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF VPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2vpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF VPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF VPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF VPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF VPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2vpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF VPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF VPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-VPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4vpu-by-release2image-of-VPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        vpu2image2xi = release2image4vpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4vpu-performed-by-release2image-of-VPU2Image4Xi: " + str(release2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; vpu2image2xi-returning-by-release2image-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return vpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-VPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF VPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF VPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2vpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF VPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF VPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF VPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2vpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF VPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF VPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF VPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-VPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4vpu-by-retain-of-VPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        vpu2image2xi = retain2image4vpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4vpu-performed-by-retain2image-of-VPU2Image4Xi: " + str(retain2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4vpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; vpu2image2xi-returning-by-retain2image-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4vpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4vpu', 
        #  'METHOD': 'create2image4vpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return vpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-VPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF VPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR VPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2vpu = {}
    #   try:
    #       response4wait2Dimage = xi2vpu.wait2image()
    #       print('(Step 122B/700) OF VPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR VPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR VPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-VPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-VPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4vpu-by-wait2image-of-VPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        vpu2image2xi = wait2image4vpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4vpu-performed-by-wait2image-of-VPU2Image4Xi: " + str(wait2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4vpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; vpu2image2xi-returning-by-wait2image-of-VPU2Image4Xi: " + str(vpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return vpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2vpu = {}
    #   try:
    #       response4write2vpu = xi2vpu.write2image()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2VPU: ', response4write2vpu) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-VPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4vpu-by-write2image-of-VPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2vpu = write2image4vpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4vpu-performed-by-write2image-of-VPU2Image4Xi: " + str(write2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4vpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2vpu-returning-by-write2image-of-VPU2Image4Xi: " + str(response4writeimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2vpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2vpu = {}
    #   try:
    #       response4copy2vpu = xi2vpu.copy2image()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2VPU: ', response4copy2vpu) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-VPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4vpu-by-copy2image-of-VPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2vpu = copy2image4vpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4vpu-performed-by-copy2image-of-VPU2Image4Xi: " + str(copy2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4vpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2vpu-returning-by-copy2image-of-VPU2Image4Xi: " + str(response4copyimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2vpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2vpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2VPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-VPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4vpu-by-copy2imagetobuffer-of-VPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2vpu = copy2imagetobuffer4vpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4vpu-performed-by-copy2imagetobuffer-of-VPU2Image4Xi: " + str(copy2imagetobuffer4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4vpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2vpu-returning-by-copy2imagetobuffer-of-VPU2Image4Xi: " + str(response4copyimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2vpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-VPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF VPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2vpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF VPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2VPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF VPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4vpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-VPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4vpu-by-copy2buffertoimage-of-VPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2vpu = copy2buffertoimage4vpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4vpu-performed-by-copy2buffertoimage-of-VPU2Image4Xi: " + str(copy2buffertoimage4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4vpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2vpu-returning-by-copy2buffertoimage-of-VPU2Image4Xi: " + str(response4copyimage2vpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2vpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-VPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF VPU2IMAGE4XI5TEST - TRYING TO TRACE VPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2vpu = None
    #   try:
    #       trace2vpu = trace2image4vpu() 
    #       print('(Step 15B/25) OF VPU2IMAGE4XI5TEST - TRACE2VPU: ', trace2vpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4vpu', 'METHOD': 'trace2image4vpu'}
    #    except:
    #       print('(Step 15Z1/25) OF VPU2IMAGE4XI5TEST *** FAILED TO TRACE VPU: ', trace2vpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-VPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4vpu-by-trace2image-of-VPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4vpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4vpu-performed-by-trace2image-of-VPU2Image4Xi: " + str(trace2image4vpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4vpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-VPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4vpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4vpu',
        #  'METHOD': 'trace2image4vpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-VPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF VPU2IMAGE4XI - TRYING TO SET VPU IMAGE')
    #   try:
    #       cc2xi = VPU2Image4Xi()                            
    #       image2vpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF VPU4XI5TEST - SET VPU IMAGE: ', image2vpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF VPU4XI5TEST *** FAILED TO SET VPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of VPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "VPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-VPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-VPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2vpu4image = {} 
        try:
            callback2vpu4image = set2imagecallback4vpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2vpu4image" + "; callback2vpu4image-set-by-set2imagecallback-of-VPU2Image4Xi to: " + str(callback2vpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4vpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2vpu4image-failed-by-set2imagecallback-of-VPU2Image4Xi: " + str(callback2vpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2vpu4image-returning-by-set2imagecallback-of-VPU2Image4Xi: " + str(callback2vpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4vpu at 0x0000000003AB31C8> 
                                             
        return callback2vpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-VPU2Image4Xi" usage = "
    #   try:
    #       print(xi2vpu.sense())
    #       print('(Step 24B/33) OF VPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses VPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'VPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; VPU2Image4Xi-sensing-by-sense-of-VPU2Image4Xi: " + str(VPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'vpu2image4xi.VPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-VPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2vpu-of-VPU2Image4Xi'] = VPU2Image4Xi.classvariable2vpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-VPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; VPU2Image4Xi-sensed-by-sense-of-VPU2Image4Xi: " + str(VPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'vpu2image4xi.VPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-VPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>, 'classvariable2vpu-of-VPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-VPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF VPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2vpu.reason())
    #       print('(Step 25B/33) OF VPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons VPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'VPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; VPU2Image4Xi-reasoning-by-reason-of-VPU2Image4Xi: " + str(VPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'vpu2image4xi.VPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-VPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-VPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-VPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; VPU2Image4Xi-reasoned-by-reason-of-VPU2Image4Xi: " + str(VPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'vpu2image4xi.VPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-VPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-VPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-VPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF VPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2vpu.ace())
    #       print('(Step 26B/33) OF VPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF VPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces VPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'VPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-VPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; VPU2Image4Xi-acing-by-ace-of-VPU2Image4Xi: " + str(VPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'vpu2image4xi.VPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-VPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-VPU2Image4Xi'] = VPU2Image4Xi.classvariable2vpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-VPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; VPU2Image4Xi-aced-by-ace-of-VPU2Image4Xi: " + str(VPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'vpu2image4xi.VPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-VPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-VPU2Image4Xi': 4, 'object8xi': <vpu2image4xi.VPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   vpu2image4xi5test.txt
#   sense-vpu2image4xi5test.xml
#   reason-vpu2image4xi5test.xml
#   ace-vpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of vpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
